import { type Schema } from 'schemata-ts/Schema';
/**
 * A trait which will throw an error for when a value does not match a schema
 *
 * @since 2.2.0
 * @category Models
 */
export interface Assert<T> {
    readonly assert: (output: unknown) => asserts output is T;
}
/**
 * A trait which will throw an error for when an expected "output" value does not match a
 * schema's output type
 *
 * @since 2.2.0
 * @category Interpreters
 */
export declare const deriveAssert: <I, O>(schema: Schema<I, O>) => Assert<O>;
/**
 * A trait which will throw an error for when an expected "input" value does not match a
 * schema's input type
 *
 * @since 2.2.0
 * @category Interpreters
 */
export declare const deriveInputAssert: <I, O>(schema: Schema<I, O>) => Assert<I>;
/**
 * A trait which will throw an error for when an expected "input" value does not match a
 * schema's input type
 *
 * Unlike `deriveInputAssert`, this will include the error tree in the error message
 *
 * @since 2.2.0
 * @category Interpreters
 */
export declare const deriveInputAssertTree: <I, O>(schema: Schema<I, O>) => Assert<I>;
//# sourceMappingURL=Assert.d.ts.map