"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deriveInputAssertTree = exports.deriveInputAssert = exports.deriveAssert = void 0;

var E = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/Either.js"));

var _Guard = /*#__PURE__*/require("schemata-ts/Guard");

var _TranscodeError = /*#__PURE__*/require("schemata-ts/TranscodeError");

var _Transcoder = /*#__PURE__*/require("schemata-ts/Transcoder");

var _TypeString = /*#__PURE__*/require("schemata-ts/TypeString");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/** @since 2.2.0 */

/**
 * A trait which will throw an error for when an expected "output" value does not match a
 * schema's output type
 *
 * @since 2.2.0
 * @category Interpreters
 */
const deriveAssert = schema => {
  const guard = (0, _Guard.deriveGuard)(schema);
  const [, name] = (0, _TypeString.deriveTypeString)(schema);
  return {
    assert: output => {
      if (!guard.is(output)) {
        throw new TypeError(`Expected ${name}, but got ${(0, _TranscodeError.safeShow)(output)}`);
      }
    }
  };
};
/**
 * A trait which will throw an error for when an expected "input" value does not match a
 * schema's input type
 *
 * @since 2.2.0
 * @category Interpreters
 */


exports.deriveAssert = deriveAssert;

const deriveInputAssert = schema => {
  const guard = (0, _Guard.deriveInputGuard)(schema);
  const [name] = (0, _TypeString.deriveTypeString)(schema);
  return {
    assert: input => {
      if (!guard.is(input)) {
        throw new TypeError(`Expected ${name}, but got ${(0, _TranscodeError.safeShow)(input)}`);
      }
    }
  };
};
/**
 * A trait which will throw an error for when an expected "input" value does not match a
 * schema's input type
 *
 * Unlike `deriveInputAssert`, this will include the error tree in the error message
 *
 * @since 2.2.0
 * @category Interpreters
 */


exports.deriveInputAssert = deriveInputAssert;

const deriveInputAssertTree = schema => {
  const transcoder = (0, _Transcoder.deriveTranscoder)(schema);
  const [name] = (0, _TypeString.deriveTypeString)(schema);
  return {
    assert: input => {
      const result = transcoder.decode(input);

      if (E.isLeft(result)) {
        throw new TypeError(`Expected ${name}, but received the following errors:\n${(0, _TranscodeError.drawTree)(result.left, {
          showHeading: false
        })}`);
      }
    }
  };
};

exports.deriveInputAssertTree = deriveInputAssertTree;
//# sourceMappingURL=Assert.js.map