"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.union = exports.tuple = exports.struct = exports.stripIdentity = exports.string = exports.ref = exports.record = exports.number = exports.nullSchema = exports.literal = exports.intersection = exports.integer = exports.emptySchema = exports.deriveJsonSchema = exports.booleanSchema = exports.array = exports.annotate = exports.URI = void 0;

var _Const = /*#__PURE__*/require("fp-ts/lib/Const.js");

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

var _jsonSchemaSchemable = /*#__PURE__*/require("schemata-ts/derivations/json-schema-schemable");

var I = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("schemata-ts/internal/json-schema"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Models for JsonSchema with derivations in Draft-07, 2019-09, and 2020-12
 *
 * @since 1.2.0
 * @see https://json-schema.org/draft/2020-12/json-schema-validation.html
 */
// -------------------------------------------------------------------------------------
// Constructors
// -------------------------------------------------------------------------------------

/**
 * Interprets a schema as a JsonSchema projecting into either Draft-07 or 2020-12
 *
 * @since 1.2.0
 * @category Interpreters
 */
const deriveJsonSchema = (schema, version = '2019-09', maintainIdentity = false) => {
  switch (version) {
    case 'Draft-07':
      // istanbul ignore next
      return (maintainIdentity ? _function.identity : I.stripIdentity)(I.asDraft07((0, _jsonSchemaSchemable.deriveJsonSchema)(schema)));

    case '2019-09':
      // istanbul ignore next
      return (maintainIdentity ? _function.identity : I.stripIdentity)((0, _jsonSchemaSchemable.deriveJsonSchema)(schema));

    case '2020-12':
      // istanbul ignore next
      return (maintainIdentity ? _function.identity : I.stripIdentity)(I.as2020((0, _jsonSchemaSchemable.deriveJsonSchema)(schema)));
  }
};
/**
 * @since 1.2.0
 * @category Constructors
 */


exports.deriveJsonSchema = deriveJsonSchema;
const emptySchema = /*#__PURE__*/(0, _Const.make)( /*#__PURE__*/new I.JsonEmpty());
/**
 * @since 1.2.0
 * @category Constructors
 */

exports.emptySchema = emptySchema;

const string = (params = {}) => (0, _Const.make)(new I.JsonString(params.minLength, params.maxLength, params.pattern, params.contentEncoding, params.contentMediaType, params.contentSchema, params.format));
/**
 * @since 1.2.0
 * @category Constructors
 */


exports.string = string;

const number = (params = {}) => (0, _Const.make)(new I.JsonNumber(params.minimum, params.maximum));
/**
 * @since 1.2.0
 * @category Constructors
 */


exports.number = number;

const integer = (params = {}) => (0, _Const.make)(new I.JsonInteger(params.minimum, params.maximum));
/**
 * @since 1.2.0
 * @category Constructors
 */


exports.integer = integer;
const booleanSchema = /*#__PURE__*/(0, _Const.make)( /*#__PURE__*/new I.JsonBoolean());
/**
 * @since 1.2.0
 * @category Constructors
 */

exports.booleanSchema = booleanSchema;

const literal = value => value === null ? (0, _Const.make)(new I.JsonNull()) : (0, _Const.make)({
  type: typeof value,
  const: value
});
/**
 * @since 1.2.0
 * @category Constructors
 */


exports.literal = literal;

const struct = (properties, required = [], additionalProperties) => (0, _Const.make)(new I.JsonStruct(properties, required, additionalProperties));
/**
 * @since 1.2.0
 * @category Constructors
 */


exports.struct = struct;

const record = (additionalProperties, propertyNames) => (0, _Const.make)(new I.JsonStruct(new I.JsonEmpty(), [], additionalProperties, propertyNames));
/**
 * @since 1.2.0
 * @category Constructors
 */


exports.record = record;

const array = (params = {}) => items => (0, _Const.make)(new I.JsonArray(items, params.minItems, params.maxItems));
/**
 * @since 1.2.0
 * @category Constructors
 */


exports.array = array;

const tuple = (...items) => (0, _Const.make)(new I.JsonArray(items, items.length, items.length));
/**
 * @since 1.2.0
 * @category Constructors
 */


exports.tuple = tuple;
const nullSchema = /*#__PURE__*/(0, _Const.make)( /*#__PURE__*/new I.JsonNull());
/**
 * @since 1.2.0
 * @category Constructors
 */

exports.nullSchema = nullSchema;

const union = (...members) => (0, _Const.make)(members.length > 1 ? new I.JsonUnion(members) : members[0]);
/**
 * @since 1.2.0
 * @category Constructors
 */


exports.union = union;

const intersection = right => left => (0, _Const.make)(new I.JsonIntersection([left, right]));
/**
 * A reference to a schema definition
 *
 * @since 2.0.0
 * @category Constructors
 */


exports.intersection = intersection;

const ref = ref => (0, _Const.make)(new I.JsonRef(ref)); // -------------------------------------------------------------------------------------
// Instances
// -------------------------------------------------------------------------------------

/**
 * @since 1.2.0
 * @category URI
 */


exports.ref = ref;
const URI = 'schemata-ts/JsonSchema'; // -------------------------------------------------------------------------------------
// Combintators
// -------------------------------------------------------------------------------------

/**
 * @since 1.2.0
 * @category Combintators
 */
// istanbul ignore next

exports.URI = URI;

const annotate = ({
  title,
  description,
  references,
  deprecated,
  readOnly
} = {}) => schema => title === undefined && description === undefined && references === undefined && deprecated === undefined && readOnly === undefined ? (0, _Const.make)(schema) : (0, _Const.make)({ ...schema,
  ...(title === undefined ? {} : {
    title
  }),
  ...(description === undefined ? {} : {
    description
  }),
  ...(references === undefined ? {} : {
    $defs: references
  }),
  ...(deprecated === undefined ? {} : {
    deprecated
  }),
  ...(readOnly === undefined ? {} : {
    readOnly
  })
}); // -------------------------------------------------------------------------------------
// Destructors
// -------------------------------------------------------------------------------------

/**
 * Removes the internal class identities from a `JsonSchema`
 *
 * @since 1.2.0
 * @category Destructors
 */


exports.annotate = annotate;
const stripIdentity = I.stripIdentity;
exports.stripIdentity = stripIdentity;
//# sourceMappingURL=JsonSchema.js.map