import type * as hkt from 'schemata-ts/internal/schemable';
import { type Schemable } from 'schemata-ts/Schemable';
declare const SchemaSymbol: unique symbol;
type SchemaSymbol = typeof SchemaSymbol;
/**
 * @since 1.0.0
 * @category Model
 */
export interface Schema<I, O = I> {
    readonly [SchemaSymbol]: SchemaSymbol;
    readonly input: (_: I) => I;
    readonly output: (_: O) => O;
    readonly runSchema: <S extends hkt.SchemableLambda>(S: Schemable<S>) => hkt.SchemableKind<S, I, O>;
}
/**
 * @since 2.1.0
 * @category Implementation
 */
export declare class SchemaImplementation<I, O = I> implements Schema<I, O> {
    /** @since 2.0.0 */
    readonly [SchemaSymbol]: SchemaSymbol;
    /** @since 2.0.0 */
    readonly input: (_: I) => I;
    /** @since 2.0.0 */
    readonly output: (_: O) => O;
    /** @since 2.0.0 */
    readonly runSchema: <S extends hkt.SchemableLambda>(S: Schemable<S>) => hkt.SchemableKind<S, I, O>;
    protected constructor(runSchema: <S extends hkt.SchemableLambda>(S: Schemable<S>) => hkt.SchemableKind<S, I, O>);
}
/**
 * @since 2.0.0
 * @category Guards
 */
export declare const isSchema: (u: unknown) => u is Schema<unknown, unknown>;
/**
 * Extract the output of a schema
 *
 * @since 1.0.0
 * @category Type Helpers
 */
export type TypeOf<S> = S extends Schema<any, infer A> ? A : never;
/**
 * Extract the output of a schema.
 *
 * Alias of `TypeOf`
 *
 * @since 1.0.0
 * @category Type Helpers
 */
export type OutputOf<S> = TypeOf<S>;
/**
 * Extract the input type of a schema.
 *
 * @since 1.0.0
 * @category Type Helpers
 */
export type InputOf<S> = S extends Schema<infer I, any> ? I : never;
/**
 * @since 1.0.0
 * @category Utilities
 */
export declare const memoize: <A, B>(f: (a: A) => B) => (a: A) => B;
export {};
//# sourceMappingURL=Schema.d.ts.map