"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.memoize = exports.isSchema = exports.SchemaImplementation = void 0;

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

/**
 * The schemata-ts schema type
 *
 * @since 1.0.0
 */
const SchemaSymbol = /*#__PURE__*/Symbol.for('schemata-ts/Schema');
/**
 * @since 2.1.0
 * @category Implementation
 */

class SchemaImplementation {
  /** @since 2.0.0 */
  [SchemaSymbol] = SchemaSymbol;
  /** @since 2.0.0 */

  input = _function.identity;
  /** @since 2.0.0 */

  output = _function.identity;
  /** @since 2.0.0 */

  runSchema;

  constructor(runSchema) {
    this.runSchema = memoize(runSchema);
  }
  /** @internal */


  static make = f => (0, _function.unsafeCoerce)(new SchemaImplementation(f));
}
/**
 * @since 2.0.0
 * @category Guards
 */


exports.SchemaImplementation = SchemaImplementation;

const isSchema = u => u !== null && typeof u === 'object' && !Array.isArray(u) && SchemaSymbol in u;
/**
 * @since 1.0.0
 * @category Utilities
 */


exports.isSchema = isSchema;

const memoize = f => {
  const cache = new Map();
  return a => {
    if (!cache.has(a)) {
      const b = f(a);
      cache.set(a, b);
      return b;
    }

    return cache.get(a);
  };
};

exports.memoize = memoize;
//# sourceMappingURL=Schema.js.map