"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.totalErrors = exports.safeShow = exports.prefixedLines = exports.foldMapDepthFirst = exports.fold = exports.drawTree = exports.draw = exports.UnexpectedValue = exports.TypeMismatch = exports.TranscodeErrors = exports.SerializationError = exports.Semigroup = exports.ErrorAtUnionMember = exports.ErrorAtKey = exports.ErrorAtIndex = void 0;

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

var N = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/number.js"));

var O = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/Option.js"));

var RA = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/ReadonlyArray.js"));

var RNEA = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/ReadonlyNonEmptyArray.js"));

var Str = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/string.js"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Represents an error ADT that occurs while encoding or decoding
 *
 * @since 2.0.0
 */

/**
 * A readonly non-empty array of transcode errors
 *
 * @since 2.0.0
 * @category Model
 */
class TranscodeErrors {
  errors;
  /** @since 2.0.0 */

  _tag = 'TranscodeErrors';

  constructor(errors) {
    this.errors = errors;
  }
  /** @since 2.2.0 */


  toJSON() {
    return draw(this);
  }
  /** @since 2.2.0 */


  toString() {
    return draw(this);
  }
  /** @since 2.2.0 */


  [Symbol.for('nodejs.util.inspect.custom')]() {
    return draw(this);
  }

}
/**
 * Represents a mismatched value
 *
 * @since 2.0.0
 * @category Model
 */


exports.TranscodeErrors = TranscodeErrors;

class TypeMismatch {
  expected;
  actual;
  /** @since 2.0.0 */

  _tag = 'TypeMismatch';

  constructor(expected, actual) {
    this.expected = expected;
    this.actual = actual;
  }

}
/**
 * Represents an error serializing or deserializing a value
 *
 * @since 2.0.0
 * @category Model
 */


exports.TypeMismatch = TypeMismatch;

class SerializationError {
  expected;
  error;
  actual;
  /** @since 2.0.0 */

  _tag = 'SerializationError';

  constructor(expected, error, actual) {
    this.expected = expected;
    this.error = error;
    this.actual = actual;
  }

}
/**
 * Represents an error for when a value is present but was not expected
 *
 * @since 2.0.0
 * @category Model
 */


exports.SerializationError = SerializationError;

class UnexpectedValue {
  actual;
  /** @since 2.0.0 */

  _tag = 'UnexpectedValue';

  constructor(actual) {
    this.actual = actual;
  }

}
/**
 * Represents an error at a specific index
 *
 * @since 2.0.0
 * @category Model
 */


exports.UnexpectedValue = UnexpectedValue;

class ErrorAtIndex {
  index;
  errors;
  /** @since 2.0.0 */

  _tag = 'ErrorAtIndex';

  constructor(index, errors) {
    this.index = index;
    this.errors = errors;
  }

}
/**
 * Represents an error at a specific key
 *
 * @since 2.0.0
 * @category Model
 */


exports.ErrorAtIndex = ErrorAtIndex;

class ErrorAtKey {
  key;
  errors;
  /** @since 2.0.0 */

  _tag = 'ErrorAtKey';

  constructor(key, errors) {
    this.key = key;
    this.errors = errors;
  }

}
/**
 * Represents an error that occurred for a particular union member
 *
 * @since 2.0.0
 * @category Model
 */


exports.ErrorAtKey = ErrorAtKey;

class ErrorAtUnionMember {
  member;
  errors;
  /** @since 2.0.0 */

  _tag = 'ErrorAtUnionMember';

  constructor(member, errors) {
    this.member = member;
    this.errors = errors;
  }

}
/**
 * @since 2.0.0
 * @category Instances
 */


exports.ErrorAtUnionMember = ErrorAtUnionMember;
const Semigroup = {
  concat: (x, y) => new TranscodeErrors(RNEA.concat(y.errors)(x.errors))
};
/**
 * Flattens a `DecodeError` tree into a common Semigroup
 *
 * @since 2.0.0
 * @category Destructors
 */

exports.Semigroup = Semigroup;

const fold = S => matchers => {
  const depthFirst = depth => err => (0, _function.pipe)(err.errors, RNEA.foldMap(S)(err => {
    switch (err._tag) {
      case 'TypeMismatch':
        return matchers.TypeMismatch(err, depth);

      case 'UnexpectedValue':
        return matchers.UnexpectedValue(err, depth);

      case 'SerializationError':
        return matchers.SerializationError(err, depth);

      case 'ErrorAtIndex':
        return matchers.ErrorAtIndex(err, depth, depthFirst(depth + 1));

      case 'ErrorAtKey':
        return matchers.ErrorAtKey(err, depth, depthFirst(depth + 1));

      case 'ErrorAtUnionMember':
        return matchers.ErrorAtUnionMember(err, depth, depthFirst(depth + 1));
    }
  }));

  return depthFirst(0);
};
/**
 * Flattens a `DecodeError` tree into a common Semigroup with access to the current
 * accumulation and current level of depth
 *
 * @since 2.0.0
 * @category Destructors
 */


exports.fold = fold;

const foldMapDepthFirst = S => matchers => fold(S)({
  TypeMismatch: (err, depth) => matchers.TypeMismatch(err.expected, err.actual, depth),
  UnexpectedValue: (err, depth) => matchers.UnexpectedValue(err.actual, depth),
  SerializationError: (err, depth) => matchers.SerializationError(err.expected, err.error, err.actual, depth),
  ErrorAtIndex: (err, depth, recurse) => matchers.ErrorAtIndex(err.index, recurse(err.errors), depth),
  ErrorAtKey: (err, depth, recurse) => matchers.ErrorAtKey(err.key, recurse(err.errors), depth),
  ErrorAtUnionMember: (err, depth, recurse) => matchers.ErrorAtUnionMember(err.member, recurse(err.errors), depth)
});
/**
 * Draws a tree of `TranscodeErrors` as lines with configurable prefix characters.
 *
 * The first argument, `prefix` appends any string or character to a concrete node. The
 * depth represents how many generations are between the root node and the current node.
 * The total children represents the total number of children the current node has. If a
 * node has zero children, it is a concrete node; i.e. type-mismatch, unexpected value, or
 * serialization error. If a node has one or more children, it is a container node; i.e.
 * error-at-key, error-at-index, or error-at-union-member.
 *
 * The second argument, `prefixChildren` appends any string or character to a child node,
 * which would have already been prefixed by the first argument. It will be called for
 * nodes that have been prefixed many times.
 *
 * The third argument, `errorStrings` is a configuration object that can be used to
 * override the default error type templates
 *
 * @since 2.0.0
 * @category Destructors
 */


exports.foldMapDepthFirst = foldMapDepthFirst;

const prefixedLines = (prefix, // istanbul ignore next
prefixChildren = (0, _function.constant)(''), errorStrings = {}) => {
  const {
    TypeMismatch = (expected, actual) => `Expected ${expected} but got ${safeShow(actual)}`,
    UnexpectedValue = actual => `Unexpected value: ${safeShow(actual)}`,
    SerializationError = (expected, error, actual) => `Expected ${expected}, but ran into serialization error: ${safeShow(error)}; got ${safeShow(actual)}`,
    ErrorAtIndex = index => `at index ${index}:`,
    ErrorAtKey = key => `at key ${String(key)}:`,
    ErrorAtUnionMember = member => `at union member \`${member}\`:`
  } = errorStrings;
  return foldMapDepthFirst(RNEA.getSemigroup())({
    TypeMismatch: (expected, actual, depth) => [`${prefix(depth, 0)}${TypeMismatch(expected, actual)}`],
    UnexpectedValue: (actual, depth) => [`${prefix(depth, 0)}${UnexpectedValue(actual)}`],
    SerializationError: (expected, error, actual, depth) => [`${prefix(depth, 0)}${SerializationError(expected, error, actual)}`],
    ErrorAtIndex: (index, errors, depth) => (0, _function.pipe)([`${prefix(depth, errors.length)}${ErrorAtIndex(index)}`], RNEA.concat((0, _function.pipe)(errors, RA.mapWithIndex((i, e) => `${prefixChildren(depth, i, errors.length, e)}${e}`)))),
    ErrorAtKey: (key, errors, depth) => (0, _function.pipe)([`${prefix(depth, errors.length)}${ErrorAtKey(key)}`], RNEA.concat((0, _function.pipe)(errors, RA.mapWithIndex((i, e) => `${prefixChildren(depth, i, errors.length, e)}${e}`)))),
    ErrorAtUnionMember: (member, errors, depth) => (0, _function.pipe)([`${prefix(depth, errors.length)}${ErrorAtUnionMember(member)}`], RNEA.concat((0, _function.pipe)(errors, RA.mapWithIndex((i, e) => `${prefixChildren(depth, i, errors.length, e)}${e}`))))
  });
};
/**
 * Returns the total number of transcode errors
 *
 * @since 2.0.0
 * @category Destructors
 */


exports.prefixedLines = prefixedLines;
const totalErrors = /*#__PURE__*/foldMapDepthFirst(N.SemigroupSum)({
  TypeMismatch: () => 1,
  UnexpectedValue: () => 1,
  SerializationError: () => 1,
  ErrorAtIndex: (_, errors) => errors,
  ErrorAtKey: (_, errors) => errors,
  ErrorAtUnionMember: (_, errors) => errors
});
/**
 * Draws a tree of `TranscodeErrors`
 *
 * @since 2.0.0
 * @category Destructors
 */

exports.totalErrors = totalErrors;

const drawTree = (errors, configuration = {}) => {
  const {
    showHeading = true
  } = configuration;
  const errorCount = totalErrors(errors);
  return (0, _function.pipe)(errors, prefixedLines((depth, totalChildren) => {
    switch (totalChildren) {
      case 0:
        return '─ ';

      default:
        return depth === 0 ? '┌ ' : '─ ';
    }
  }, (depth, childIndex, totalChildren) => {
    switch (depth) {
      case 0:
        return childIndex === totalChildren - 1 ? '└─' : '├─';

      default:
        return '─';
    }
  }), RA.intercalate(Str.Monoid)('\n'), _ => `${showHeading ? `Encountered ${errorCount} transcode error${errorCount === 1 ? '' : 's'}:\n` : ''}${_}`);
};
/**
 * Alias for `drawTree`
 *
 * @since 2.0.0
 * @category Destructors
 */


exports.drawTree = drawTree;
const draw = drawTree;
/** @internal */

exports.draw = draw;

const typeOf = a => {
  if (Array.isArray(a)) return 'Array';
  return typeof a;
};
/** @internal */


const safeShow = a => {
  if (a instanceof Error || typeof a === 'symbol' || typeof a === 'number' && (Number.isNaN(a) || !Number.isFinite(a))) return String(a);
  if (typeof a === 'string') return `"${a}"`;
  if (typeof a === 'bigint') return `${a}n`;
  if (a instanceof Map) return `Map(${a.size})`;
  if (a instanceof Set) return `Set(${a.size})`;
  if (typeof a === 'function') return `[Function ${a.name === '' ? '(anonymous)' : a.name}]`;
  const tA = typeOf(a);
  return (0, _function.pipe)(O.tryCatch(() => {
    const seen = new WeakSet();
    return JSON.stringify(a, (_, v) => {
      if (typeof v === 'object' && v !== null) {
        if (seen.has(v)) return `[Circular ${tA}]`;
        seen.add(v);
      }

      return typeof v === 'bigint' ? `${v}n` : v;
    });
  }), // unreachable case, such as if a circular object does not have a constant reference (???)
  O.getOrElse( // istanbul ignore next
  () => `[Circular ${tA}]`));
};

exports.safeShow = safeShow;
//# sourceMappingURL=TranscodeError.js.map