"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.unexpectedValue = exports.typeMismatch = exports.transcodeErrors = exports.success = exports.serializationError = exports.imap = exports.failure = exports.errorAtUnionMember = exports.errorAtKey = exports.errorAtIndex = exports.drawErrorTree = exports.deriveTranscoder = exports.URI = exports.Invariant = void 0;

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

var _transcoderSchemable = /*#__PURE__*/require("schemata-ts/derivations/transcoder-schemable");

var I = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("schemata-ts/internal/transcoder"));

var TE = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("schemata-ts/TranscodeError"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------

/**
 * @since 2.0.0
 * @category Constructors
 */
const success = I.success;
/**
 * @since 2.0.0
 * @category Constructors
 */

exports.success = success;
const failure = I.failure;
/**
 * A collection of failure cases
 *
 * @since 2.0.0
 * @category Constructors
 */

exports.failure = failure;
const transcodeErrors = I.transcodeErrors;
/**
 * A failure case for a value that does not match the expected type
 *
 * @since 2.0.0
 * @category Constructors
 */

exports.transcodeErrors = transcodeErrors;
const typeMismatch = I.typeMismatch;
/**
 * A failure case for an unexpected value
 *
 * @since 2.0.0
 * @category Constructors
 */

exports.typeMismatch = typeMismatch;
const unexpectedValue = I.unexpectedValue;
/**
 * A failure case for a serialization or deserialization error
 *
 * @since 2.0.0
 * @category Constructors
 */

exports.unexpectedValue = unexpectedValue;
const serializationError = I.serializationError;
/**
 * A failure case at a specific index
 *
 * @since 2.0.0
 * @category Constructors
 */

exports.serializationError = serializationError;

const errorAtIndex = (i, ...errs) => I.errorAtIndex(i, transcodeErrors(...errs));
/**
 * A failure case at a specific key
 *
 * @since 2.0.0
 * @category Constructors
 */


exports.errorAtIndex = errorAtIndex;

const errorAtKey = (key, ...errs) => I.errorAtKey(key, transcodeErrors(...errs));
/**
 * A failure case for a union member
 *
 * @since 2.0.0
 * @category Constructors
 */


exports.errorAtKey = errorAtKey;

const errorAtUnionMember = (member, ...errs) => I.errorAtUnionMember(member, transcodeErrors(...errs)); // -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------

/**
 * @since 2.0.0
 * @category Destructors
 */


exports.errorAtUnionMember = errorAtUnionMember;
const drawErrorTree = TE.drawTree; // -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------

/**
 * Interprets a schema as a decoder
 *
 * @since 2.0.0
 * @category Interpreters
 */

exports.drawErrorTree = drawErrorTree;
const deriveTranscoder = /*#__PURE__*/(0, _function.unsafeCoerce)(_transcoderSchemable.deriveTranscoder); // -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------

/**
 * @since 2.0.0
 * @category URI
 */

exports.deriveTranscoder = deriveTranscoder;
const URI = 'schemata-ts/Transcoder';
/**
 * @since 2.0.0
 * @category Instance Methods
 */

exports.URI = URI;
const imap = /*#__PURE__*/(0, _function.unsafeCoerce)(I.imap);
/**
 * @since 2.0.0
 * @category Instances
 */
// istanbul ignore next

exports.imap = imap;
const Invariant = {
  URI,
  imap: (fa, f, g) => (0, _function.pipe)(fa, imap(f, g))
};
exports.Invariant = Invariant;
//# sourceMappingURL=Transcoder.js.map