/**
 * `Guard` is a data-type that narrows an unknown value to a specific type using
 * Typescript type guards
 *
 * @since 2.0.0
 */
import * as E from "fp-ts/es6/Either.js";
import { deriveTranscoder } from 'schemata-ts/derivations/transcoder-schemable';
import * as I from 'schemata-ts/internal/guard'; // ------------------
// constructors
// ------------------

/**
 * Constructs a guard from predicate function
 *
 * @since 2.0.0
 * @category Constructors
 */

export const fromPredicate = I.fromPredicate; // ------------------
// combinators
// ------------------

export {
/**
 * Interprets a schema as a guard for the `Output` type
 *
 * @since 2.0.0
 * @category Interpreters
 */
deriveGuard } from 'schemata-ts/derivations/guard-schemable';
/**
 * Interprets a schema as a guard for the `Input` type
 *
 * @since 2.1.0
 * @category Interpreters
 */

export const deriveInputGuard = schema => {
  const transcoder = deriveTranscoder(schema);
  return {
    is: u => E.isRight(transcoder.decode(u))
  };
}; // ------------------
// instances
// ------------------

/**
 * @since 2.0.0
 * @category URI
 */

export const URI = 'schemata-ts/Guard';
//# sourceMappingURL=Guard.mjs.map