/**
 * The schemata-ts schema type
 *
 * @since 1.0.0
 */
import { identity, unsafeCoerce } from "fp-ts/es6/function.js";
const SchemaSymbol = /*#__PURE__*/Symbol.for('schemata-ts/Schema');
/**
 * @since 2.1.0
 * @category Implementation
 */

export class SchemaImplementation {
  /** @since 2.0.0 */
  [SchemaSymbol] = SchemaSymbol;
  /** @since 2.0.0 */

  input = identity;
  /** @since 2.0.0 */

  output = identity;
  /** @since 2.0.0 */

  runSchema;

  constructor(runSchema) {
    this.runSchema = memoize(runSchema);
  }
  /** @internal */


  static make = f => unsafeCoerce(new SchemaImplementation(f));
}
/**
 * @since 2.0.0
 * @category Guards
 */

export const isSchema = u => u !== null && typeof u === 'object' && !Array.isArray(u) && SchemaSymbol in u;
/**
 * @since 1.0.0
 * @category Utilities
 */

export const memoize = f => {
  const cache = new Map();
  return a => {
    if (!cache.has(a)) {
      const b = f(a);
      cache.set(a, b);
      return b;
    }

    return cache.get(a);
  };
};
//# sourceMappingURL=Schema.mjs.map