import { pipe, unsafeCoerce } from "fp-ts/es6/function.js";
import { deriveTranscoder as deriveTranscoder_ } from 'schemata-ts/derivations/transcoder-schemable';
import * as I from 'schemata-ts/internal/transcoder';
import * as TE from 'schemata-ts/TranscodeError'; // -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------

/**
 * @since 2.0.0
 * @category Constructors
 */

export const success = I.success;
/**
 * @since 2.0.0
 * @category Constructors
 */

export const failure = I.failure;
/**
 * A collection of failure cases
 *
 * @since 2.0.0
 * @category Constructors
 */

export const transcodeErrors = I.transcodeErrors;
/**
 * A failure case for a value that does not match the expected type
 *
 * @since 2.0.0
 * @category Constructors
 */

export const typeMismatch = I.typeMismatch;
/**
 * A failure case for an unexpected value
 *
 * @since 2.0.0
 * @category Constructors
 */

export const unexpectedValue = I.unexpectedValue;
/**
 * A failure case for a serialization or deserialization error
 *
 * @since 2.0.0
 * @category Constructors
 */

export const serializationError = I.serializationError;
/**
 * A failure case at a specific index
 *
 * @since 2.0.0
 * @category Constructors
 */

export const errorAtIndex = (i, ...errs) => I.errorAtIndex(i, transcodeErrors(...errs));
/**
 * A failure case at a specific key
 *
 * @since 2.0.0
 * @category Constructors
 */

export const errorAtKey = (key, ...errs) => I.errorAtKey(key, transcodeErrors(...errs));
/**
 * A failure case for a union member
 *
 * @since 2.0.0
 * @category Constructors
 */

export const errorAtUnionMember = (member, ...errs) => I.errorAtUnionMember(member, transcodeErrors(...errs)); // -------------------------------------------------------------------------------------
// destructors
// -------------------------------------------------------------------------------------

/**
 * @since 2.0.0
 * @category Destructors
 */

export const drawErrorTree = TE.drawTree; // -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------

/**
 * Interprets a schema as a decoder
 *
 * @since 2.0.0
 * @category Interpreters
 */

export const deriveTranscoder = /*#__PURE__*/unsafeCoerce(deriveTranscoder_); // -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------

/**
 * @since 2.0.0
 * @category URI
 */

export const URI = 'schemata-ts/Transcoder';
/**
 * @since 2.0.0
 * @category Instance Methods
 */

export const imap = /*#__PURE__*/unsafeCoerce(I.imap);
/**
 * @since 2.0.0
 * @category Instances
 */
// istanbul ignore next

export const Invariant = {
  URI,
  imap: (fa, f, g) => pipe(fa, imap(f, g))
};
//# sourceMappingURL=Transcoder.mjs.map