import * as E from "fp-ts/es6/Either.js";
import { identity, pipe } from "fp-ts/es6/function.js";
import * as RR from "fp-ts/es6/ReadonlyRecord.js";
import * as T from "fp-ts/es6/Task.js";
import * as TE from "fp-ts/es6/TaskEither.js";
/* istanbul ignore next */

/** @internal */

export const base64Encode = s => {
  try {
    return Buffer ? Buffer.from(s).toString('base64') : btoa(s);
  } catch (_) {
    return btoa(s);
  }
}; // istanbul ignore next

/** @internal */

export const base64Decode = s => {
  try {
    return Buffer ? Buffer.from(s, 'base64').toString() : atob(s);
  } catch (_) {
    return atob(s);
  }
};
/* istanbul ignore next */

/**
 * Object.hasOwn isn't available in Node 14 :(
 *
 * @internal
 */
// eslint-disable-next-line @typescript-eslint/ban-types

export const hasOwn = (o, v) => Object.hasOwn ? Object.hasOwn(o, v) : Object.prototype.hasOwnProperty.call(o, v);
/**
 * Performs a validative traversal over a struct's own enumerable properties while mapping
 * output types.
 *
 * @internal
 */

export const witherRemap = sgErrors => f => s => {
  const effects = {};
  /* Enumerable own, Enumerable inherited */

  for (const key in s) {
    /* Ignores inherited properties */
    // istanbul ignore next
    if (!hasOwn(s, key)) continue;
    /* Perform effect */

    effects[key] = f(key, s[key]);
  }

  return pipe(effects, RR.wither(E.getApplicativeValidation(sgErrors))(identity), E.map(s => {
    const out = {};

    for (const key in s) {
      // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
      const [value, newKey, semigroup] = s[key];

      if (hasOwn(out, newKey)) {
        out[newKey] = semigroup.concat(out[newKey], value);
        continue;
      }

      out[newKey] = value;
    }

    return out;
  }));
};

const getApplicativeValidationPar = sgErrors => TE.getApplicativeTaskValidation(T.ApplicativePar, sgErrors);
/** Performs a task-validative traversal over a struct's own enumerable properties. */


export const witherRemapPar = sgErrors => f => s => {
  const effects = {};
  /* Enumerable own, Enumerable inherited */

  for (const key in s) {
    /* Ignores inherited properties */
    // istanbul ignore next
    if (!hasOwn(s, key)) continue;
    /* Perform effect */

    effects[key] = f(key, s[key]);
  }

  return pipe(effects, RR.wither(getApplicativeValidationPar(sgErrors))(identity), TE.map(s => {
    const out = {};

    for (const key in s) {
      // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
      const [value, newKey, semigroup] = s[key];

      if (hasOwn(out, newKey)) {
        out[newKey] = semigroup.concat(out[newKey], value);
        continue;
      }

      out[newKey] = value;
    }

    return out;
  }));
};
//# sourceMappingURL=util.mjs.map