import { altAll } from "fp-ts/es6/Alt.js";
import * as E from "fp-ts/es6/Either.js";
import { pipe } from "fp-ts/es6/function.js";
import * as RA from "fp-ts/es6/ReadonlyArray.js";
import * as RNEA from "fp-ts/es6/ReadonlyNonEmptyArray.js";
import * as TC from 'schemata-ts/internal/transcoder';
import { ordGuardedPrecedentedUnionMember } from 'schemata-ts/schemables/guarded-union/definition';
import * as TCE from 'schemata-ts/TranscodeError';
const altValidation = /*#__PURE__*/E.getAltValidation(TCE.Semigroup);
export const GuardedUnionTranscoder = {
  guardedUnion: (...members) => {
    const sortedMembers = pipe(members, RNEA.sort(ordGuardedPrecedentedUnionMember));
    const firstMember = RNEA.head(sortedMembers);
    const tailMembers = RNEA.tail(sortedMembers);
    return {
      encode: out => {
        for (const m of sortedMembers) {
          const {
            guard,
            member
          } = m;

          if (guard.is(out)) {
            return member.encode(out);
          }
        }

        return pipe(sortedMembers, RNEA.map(({
          name
        }) => TC.errorAtUnionMember(name, TC.transcodeErrors(TC.typeMismatch(name, out)))), errs => TC.transcodeErrors(...errs), TC.failure);
      },
      decode: u => pipe(tailMembers, RA.map(_ => pipe(_.member.decode(u), E.mapLeft(errors => TC.transcodeErrors(TC.errorAtUnionMember(_.name, errors))))), altAll(altValidation)(pipe(firstMember.member.decode(u), E.mapLeft(errors => sortedMembers.length > 1 ? TC.transcodeErrors(TC.errorAtUnionMember(firstMember.name, errors)) : errors))))
    };
  }
};
//# sourceMappingURL=transcoder.mjs.map