import { flow } from "fp-ts/es6/function.js";
import * as G from 'schemata-ts/internal/guard';
import { isFloat, isInt } from 'schemata-ts/schemables/primitives/utils';

const isString = s => typeof s === 'string';

const isBoolean = b => typeof b === 'boolean';

const isNumber = n => typeof n === 'number';

const isNull = n => n === null;

export const PrimitivesGuard = {
  string: (params = {}) => ({
    is: s => isString(s) && (params.minLength === undefined || s.length >= params.minLength) && (params.maxLength === undefined || s.length <= params.maxLength)
  }),
  int: /*#__PURE__*/flow(isInt, G.fromPredicate),
  float: /*#__PURE__*/flow(isFloat, G.fromPredicate),
  boolean: /*#__PURE__*/G.fromPredicate(isBoolean),
  unknown: {
    is: _ => true
  },
  literal: (...literals) => ({
    is: u => (isString(u) || isNumber(u) || isBoolean(u) || isNull(u)) && literals.includes(u)
  })
};
//# sourceMappingURL=guard.mjs.map