import * as Eq from "fp-ts/es6/Eq.js";
import { pipe } from "fp-ts/es6/function.js";
import * as RA from "fp-ts/es6/ReadonlyArray.js";
import * as Inf from 'schemata-ts/internal/information';
export const PrimitivesInformation = {
  string: (params = {}) => {
    const {
      minLength = Number.MIN_SAFE_INTEGER,
      maxLength = Number.MAX_SAFE_INTEGER
    } = params;
    return Inf.informationFromSampleSize(maxLength - minLength);
  },
  int: (params = {}) => {
    const {
      min = Number.MIN_SAFE_INTEGER,
      max = Number.MAX_SAFE_INTEGER
    } = params;
    return Inf.informationFromSampleSize(max - min);
  },
  float: (params = {}) => {
    const {
      min = -Number.MAX_VALUE,
      max = Number.MAX_VALUE
    } = params;
    const sampleSize = max - min;
    return Number.isFinite(sampleSize) ? Inf.informationFromSampleSize(sampleSize) : Inf.makeInformation(1024);
  },
  boolean: /*#__PURE__*/Inf.informationFromSampleSize(2),
  unknown: /*#__PURE__*/Inf.makeInformation(Number.MAX_SAFE_INTEGER),
  literal: (...items) => pipe(items, RA.uniq(Eq.eqStrict), RA.size, unique => Inf.informationFromSampleSize(unique))
};
//# sourceMappingURL=information.mjs.map