import * as B from "fp-ts/es6/boolean.js";
import { pipe } from "fp-ts/es6/function.js";
import * as Pred from "fp-ts/es6/Predicate.js";
import * as RR from "fp-ts/es6/ReadonlyRecord.js";
import * as Str from "fp-ts/es6/string.js";
import { remapPropertyKeys } from 'schemata-ts/schemables/struct/utils';

const isObject = u => typeof u === 'object' && u !== null && !Array.isArray(u);

export const StructGuard = {
  struct: (properties, // istanbul ignore next
  extraProps = 'strip') => {
    const lookupByOutputKey = remapPropertyKeys(properties);
    return {
      is: pipe(isObject, Pred.and(u => {
        // -- if extra props are not allowed, return a failure on keys not specified in properties
        if (extraProps === 'error') {
          for (const key in u) {
            if (lookupByOutputKey[key] === undefined) return false;
          }
        } // -- if extra props are stripped then their presence is not a failure


        const knownMatch = pipe(lookupByOutputKey, RR.foldMapWithIndex(Str.Ord)(B.MonoidAll)((key, members) => {
          const inputAtKey = u[key];

          for (const {
            member
          } of members) {
            if (member.is(inputAtKey)) return true;
          }

          return false;
        }));

        if (typeof extraProps === 'string' || knownMatch === false) {
          return knownMatch;
        }

        const restMatch = pipe(u, RR.foldMapWithIndex(Str.Ord)(B.MonoidAll)((key, value) => {
          if (lookupByOutputKey[key] !== undefined) return true;
          return extraProps.is(value);
        }));
        return restMatch && knownMatch;
      }), a => a)
    };
  },
  record: (sk, so) => ({
    is: pipe(isObject, Pred.and(RR.foldMapWithIndex(Str.Ord)(B.MonoidAll)((key, value) => sk.is(key) && so.is(value))), _ => _)
  }),
  intersection: (x, y) => ({
    is: pipe(x.is, Pred.and(y.is), _ => _)
  })
};
//# sourceMappingURL=guard.mjs.map