import { identity, pipe } from "fp-ts/es6/function.js";
import * as O from "fp-ts/es6/Option.js";
import * as RA from "fp-ts/es6/ReadonlyArray.js";
import * as RNEA from "fp-ts/es6/ReadonlyNonEmptyArray.js";
import * as RR from "fp-ts/es6/ReadonlyRecord.js";
import { getKeyRemap } from 'schemata-ts/internal/struct';
import * as TC from 'schemata-ts/internal/transcoder';
import { hasOwn } from 'schemata-ts/internal/util';
import { ordGuardedPrecedentedUnionMember } from 'schemata-ts/schemables/guarded-union/definition';
/**
 * Collapses a struct definition into a record with remapped keys whose values are union
 * members of every output key collision for that key.
 */

export const remapPropertyKeys = (properties, mapInformation = identity) => {
  // --- a lookup table of output keys to union items
  const lookupByOutputKey_ = {};

  for (const key in properties) {
    // -- ignore inherited properties
    // istanbul ignore next
    if (!hasOwn(properties, key)) continue; // eslint-disable-next-line @typescript-eslint/no-non-null-assertion

    const {
      schemable,
      guard,
      information,
      semigroup,
      name
    } = properties[key];
    const inputKey = key;
    const outputKey = pipe(getKeyRemap(schemable), O.getOrElse(() => inputKey));
    const nextUnionItem = {
      member: schemable,
      guard,
      precedence: mapInformation(information),
      inputKey,
      semigroup,
      name
    };
    const outputKeyUnion = pipe(lookupByOutputKey_, RR.lookup(outputKey), O.fold(() => RNEA.of(nextUnionItem), RA.append(nextUnionItem)));
    lookupByOutputKey_[outputKey] = outputKeyUnion;
  }

  return pipe(lookupByOutputKey_, RR.map(RNEA.sort(ordGuardedPrecedentedUnionMember)));
};
export const safeIntersect = (a, b) => {
  const isObjectA = typeof a === 'object' && a !== null;
  const isObjectB = typeof b === 'object' && b !== null;
  if (isObjectA || isObjectB) return Object.assign({}, a, b);
  return void 0;
};
export const getValidateObject = M => name => u => {
  if (u === null || typeof u !== 'object' || Array.isArray(u)) {
    return M.throwError(TC.transcodeErrors(TC.typeMismatch(name, u)));
  }

  return M.of(u);
};
//# sourceMappingURL=utils.mjs.map