/** @since 1.2.0 */
import { pipe } from "fp-ts/es6/function.js";
import * as RR from "fp-ts/es6/ReadonlyRecord.js";
import { deriveJsonSchema } from 'schemata-ts/derivations/json-schema-schemable';
import { make } from 'schemata-ts/internal/schema';
/**
 * Annotate a Json Schema with title, description, and references.
 *
 * Note: references must be specified for all "Lazy" schemas, and must occur after the
 * declaration of the primary schema. This is because mutual-recursion for json-schema is
 * implemented as a ref, and that ref must be specified in the parent using `Annotate`
 *
 * @since 1.2.0
 * @category Combinators
 */

export const Annotate = params => schema => {
  const {
    references = {}
  } = params;
  const mappedRefs = pipe(references, RR.map(deriveJsonSchema));
  return make(s => s.annotate({ ...(params.title === undefined ? {} : {
      title: params.title
    }),
    ...(params.description === undefined ? {} : {
      description: params.description
    }),
    ...(params.references === undefined ? {} : {
      references: mappedRefs
    }),
    ...(params.typeString === undefined ? {} : {
      typeString: params.typeString
    }),
    ...(params.readOnly === undefined ? {} : {
      readOnly: params.readOnly
    }),
    ...(params.deprecated === undefined ? {} : {
      deprecated: params.deprecated
    })
  })(schema.runSchema(s)));
};
//# sourceMappingURL=Annotate.mjs.map