/** @since 1.4.0 */
import { pipe, unsafeCoerce } from "fp-ts/es6/function.js";
import { deriveTypeString } from 'schemata-ts/derivations/type-string-schemable';
import { SchemaImplementation } from 'schemata-ts/Schema';
import { ArrayTypeString } from 'schemata-ts/schemables/array/instances/type-string';
/**
 * An array type of known values.
 *
 * @since 1.0.0
 * @category Combinators
 */

export const Array = (codomain, params) => new ArraySchema(codomain, params);
/**
 * The ArraySchema schema transformer, to construct an ArraySchema use `S.Array` combinator instead.
 *
 * @since 2.2.0
 * @category Transformations
 */

export class ArraySchema extends SchemaImplementation {
  codomain;
  params;

  constructor(codomain, params = {}) {
    const {
      errorName: errorName_
    } = params;
    const expectedName = errorName_ ?? pipe(deriveTypeString(codomain), ArrayTypeString.array({ ...params,
      expectedName: ''
    }))[0];
    super(_ => _.array({ ...params,
      expectedName
    })(codomain.runSchema(_)));
    this.codomain = codomain;
    this.params = params;
  }
  /**
   * Requires a minimum array length
   *
   * Returns a new ArraySchema
   *
   * @since 2.2.0
   */


  minLength = minLength => new ArraySchema(this.codomain, { ...this.params,
    minLength
  });
  /**
   * Requires a maximum array length
   *
   * Returns a new ArraySchema
   *
   * @since 2.2.0
   */

  maxLength = maxLength => new ArraySchema(this.codomain, { ...this.params,
    maxLength
  });
  /**
   * Converts an array schema to a non-empty array schema
   *
   * Returns a new Schema
   *
   * @since 2.2.0
   */

  nonEmpty = () => unsafeCoerce(this.minLength(1));
}
//# sourceMappingURL=Array.mjs.map