/** @since 1.0.0 */
import { flow, pipe } from "fp-ts/es6/function.js";
import * as O from "fp-ts/es6/Option.js";
import * as Str from "fp-ts/es6/string.js";
import * as k from 'kuvio';
import { Brand } from 'schemata-ts/schemata/Brand';
import { Imap } from 'schemata-ts/schemata/Imap';
import { Pattern } from 'schemata-ts/schemata/Pattern';
import { Refine } from 'schemata-ts/schemata/Refine';
/**
 * @since 1.0.0
 * @category Pattern
 */

const binaryBigIntString = /*#__PURE__*/pipe( /*#__PURE__*/k.exactString('0b'), /*#__PURE__*/k.andThen( /*#__PURE__*/pipe( /*#__PURE__*/k.characterClass(false, ['0', '1']), /*#__PURE__*/k.atLeastOne())));
/**
 * @since 1.0.0
 * @category Pattern
 */

const octalBigIntString = /*#__PURE__*/pipe( /*#__PURE__*/k.exactString('0o'), /*#__PURE__*/k.andThen( /*#__PURE__*/pipe( /*#__PURE__*/k.characterClass(false, ['0', '7']), /*#__PURE__*/k.atLeastOne())));
/**
 * @since 1.0.0
 * @category Pattern
 */

const decimalBigIntString = /*#__PURE__*/pipe( /*#__PURE__*/k.char('-'), k.maybe, /*#__PURE__*/k.andThen( /*#__PURE__*/pipe(k.digit, /*#__PURE__*/k.atLeastOne())));
/**
 * @since 1.0.0
 * @category Pattern
 */

const hexBigIntString = /*#__PURE__*/pipe( /*#__PURE__*/k.exactString('0x'), /*#__PURE__*/k.andThen( /*#__PURE__*/pipe(k.hexDigit, /*#__PURE__*/k.atLeastOne())));
/**
 * @since 1.0.0
 * @category Pattern
 */

export const bigIntString = /*#__PURE__*/k.oneOf(binaryBigIntString, octalBigIntString, decimalBigIntString, hexBigIntString);
/**
 * Represents bigints converted from strings
 *
 * @since 1.0.0
 * @category Conversion
 */

export const BigIntFromString = /*#__PURE__*/pipe( /*#__PURE__*/Pattern(bigIntString, 'BigIntString'), /*#__PURE__*/Refine(s => pipe(s, O.fromPredicate(flow(Str.trim, s => s.length > 0)), O.chain(O.tryCatchK(s => BigInt(s))), O.isSome), 'bigint'), /*#__PURE__*/Brand(), /*#__PURE__*/Imap({
  is: u => typeof u === 'bigint'
}, b => BigInt(b), n => n.toString()));
//# sourceMappingURL=BigIntFromString.mjs.map