/** @since 1.0.0 */
import { pipe } from "fp-ts/es6/function.js";
import * as k from 'kuvio';
import { Brand } from 'schemata-ts/schemata/Brand';
import { Pattern } from 'schemata-ts/schemata/Pattern';
/**
 * /^(bc1)[a-z0-9]{25,39}$/
 *
 * @internal
 */

const bech32 = /*#__PURE__*/pipe( /*#__PURE__*/k.exactString('bc1'), /*#__PURE__*/k.andThen( /*#__PURE__*/pipe( /*#__PURE__*/k.characterClass(false, ['a', 'z'], ['0', '9']), /*#__PURE__*/k.between(25, 39))));
/**
 * /^(1|3)[A-HJ-NP-Za-km-z1-9]{25,39}$/
 *
 * @internal
 */

const base58 = /*#__PURE__*/pipe( /*#__PURE__*/k.subgroup( /*#__PURE__*/k.characterClass(false, ['1', '3'])), /*#__PURE__*/k.andThen( /*#__PURE__*/pipe( /*#__PURE__*/k.characterClass(false, ['A', 'H'], ['J', 'N'], ['P', 'Z'], ['a', 'k'], ['m', 'z'], ['1', '9']), /*#__PURE__*/k.between(25, 39))));
/**
 * @since 1.0.0
 * @category Pattern
 */

export const bitcoinAddress = /*#__PURE__*/pipe(bech32, /*#__PURE__*/k.or(base58));
/**
 * Represents strings which are valid Bitcoin addresses.
 *
 * @since 1.0.0
 * @category String
 */

export const BitcoinAddress = /*#__PURE__*/Brand()( /*#__PURE__*/Pattern(bitcoinAddress, 'BitcoinAddress'));
//# sourceMappingURL=BitcoinAddress.mjs.map