/** @since 1.0.0 */
import { pipe } from "fp-ts/es6/function.js";
import { deriveGuard } from 'schemata-ts/derivations/guard-schemable';
import { Boolean } from 'schemata-ts/schemata/Boolean';
import { Float } from 'schemata-ts/schemata/Float';
import { Imap } from 'schemata-ts/schemata/Imap';
/**
 * A boolean value whose encoded representation is a number, where 0 is false and 1 is true.
 *
 * Strictly speaking, this will _decode_ any number (0 becomes false, and non-zero becomes
 * true), but will only _encode_ 0 and 1.
 *
 * @since 1.0.0
 * @category Conversion
 */

export const BooleanFromNumber = /*#__PURE__*/pipe( /*#__PURE__*/Float(), /*#__PURE__*/Imap( /*#__PURE__*/deriveGuard(Boolean), n => n !== 0, b => b ? 1 : 0, 'boolean'));
//# sourceMappingURL=BooleanFromNumber.mjs.map