/** @since 1.4.0 */
import { deriveGuard } from 'schemata-ts/derivations/guard-schemable';
import { deriveInformation } from 'schemata-ts/derivations/information-schemable';
import { deriveMergeSemigroup } from 'schemata-ts/derivations/merge-semigroup-schemable';
import { deriveTypeString } from 'schemata-ts/derivations/type-string-schemable';
import { camelCase } from 'schemata-ts/internal/camelcase';
import { make } from 'schemata-ts/internal/schema';
import { remapKey } from 'schemata-ts/internal/struct';
import { StructTypeString } from 'schemata-ts/schemables/struct/instances/type-string';
/**
 * The same as the `Struct` schema combinator, but keys are transformed to camel case in
 * the output type.
 *
 * **Warning** It is possible to have one or more input keys map to the same output key.
 * This combination will produce unlawful instances.
 *
 * @since 1.4.0
 * @category Combinators
 */

export const CamelCaseKeys = (props, extraProps = 'strip', mergeStrategy) => make(_ => {
  const struct = {};
  const structName = {};

  for (const key in props) {
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    const schema = props[key];
    const schemable = remapKey(schema.runSchema(_), _.clone, camelCase(key));
    const guard = deriveGuard(schema);
    const information = deriveInformation(schema);
    const name = deriveTypeString(schema);
    const semigroup = deriveMergeSemigroup(schema).semigroup(mergeStrategy);
    struct[key] = {
      schemable,
      guard,
      information,
      semigroup,
      name: name[0]
    };
    structName[key] = {
      schemable: name,
      guard,
      information,
      semigroup,
      name: name[0]
    };
  }

  const wholeName = StructTypeString.struct(structName, extraProps, '');
  return _.struct(struct, extraProps, wholeName[0]);
});
//# sourceMappingURL=CamelCaseKeys.mjs.map