/** @since 1.0.0 */
import { pipe } from "fp-ts/es6/function.js";
import { deriveGuard } from 'schemata-ts/derivations/guard-schemable';
import { deriveTypeString } from 'schemata-ts/derivations/type-string-schemable';
import { Date as DateS } from 'schemata-ts/schemata/Date';
import { Float as FloatS } from 'schemata-ts/schemata/Float';
import { Imap } from 'schemata-ts/schemata/Imap';
/** @since 2.0.0 */

export const minUnixTime = -8_640_000_000_000;
/** @since 2.0.0 */

export const maxUnixTime = 8_640_000_000_000;
const UnixTimeFloat = /*#__PURE__*/FloatS({
  min: minUnixTime,
  max: maxUnixTime
});
const DateSchema = /*#__PURE__*/DateS();
/**
 * Represents Date objects derived from unix time.
 *
 * @since 1.0.0
 * @category Conversion
 */

export const DateFromUnixTime = /*#__PURE__*/pipe(UnixTimeFloat, /*#__PURE__*/Imap( /*#__PURE__*/deriveGuard(DateSchema), n => new Date(n * 1000), d => d.getTime() / 1000, /*#__PURE__*/deriveTypeString(DateSchema)[1]));
//# sourceMappingURL=DateFromUnixTime.mjs.map