/** @since 1.0.0 */
import { pipe } from "fp-ts/es6/function.js";
import * as k from 'kuvio';
import { PrimitivesGuard } from 'schemata-ts/schemables/primitives/instances/guard';
import { PrimitivesTypeString } from 'schemata-ts/schemables/primitives/instances/type-string';
import { Brand } from 'schemata-ts/schemata/Brand';
import { Imap } from 'schemata-ts/schemata/Imap';
import { Pattern } from 'schemata-ts/schemata/Pattern';
/**
 * @since 1.0.0
 * @category Pattern
 */

const binaryIntString = /*#__PURE__*/pipe( /*#__PURE__*/k.exactString('0b'), /*#__PURE__*/k.andThen( /*#__PURE__*/pipe( /*#__PURE__*/k.characterClass(false, ['0', '1']), /*#__PURE__*/k.between(1, 53))));
/**
 * @since 1.0.0
 * @category Pattern
 */

const octalIntString = /*#__PURE__*/pipe( /*#__PURE__*/k.exactString('0o'), /*#__PURE__*/k.andThen( /*#__PURE__*/pipe( /*#__PURE__*/k.characterClass(false, ['0', '7']), /*#__PURE__*/k.between(1, 18))));
/**
 * @since 1.0.0
 * @category Pattern
 */

const decimalIntString = /*#__PURE__*/pipe( /*#__PURE__*/k.char('-'), k.maybe, /*#__PURE__*/k.andThen( /*#__PURE__*/pipe(k.digit, /*#__PURE__*/k.between(1, 16))));
/**
 * @since 1.0.0
 * @category Pattern
 */

const hexIntString = /*#__PURE__*/pipe( /*#__PURE__*/k.exactString('0x'), /*#__PURE__*/k.andThen( /*#__PURE__*/pipe(k.hexDigit, /*#__PURE__*/k.between(1, 14))));
/**
 * @since 1.0.0
 * @category Pattern
 */

export const intFromString = /*#__PURE__*/k.oneOf(binaryIntString, octalIntString, decimalIntString, hexIntString);
/**
 * Integer branded newtype from string. Parameters: min, max are inclusive.
 *
 * Note: has an optional `encodeToBase` parameter that controls the output base of the
 * encoded string. Currently only decodes binary, octal, decimal, and hexadecimal input
 * bases. It decodes in any base, and encodes to supplied parameter defaulting to decimal.
 *
 * _Note_: Does not currently allow exponential notation (e.g. `0x123e4`).
 *
 * Represents string-integers:
 *
 * ```math
 *  { z | z ∈ ℤ, z >= -2 ** 53 + 1, z <= 2 ** 53 - 1 }
 * ```
 *
 * @deprecated Use `ParseInt` instead.
 * @since 1.0.0
 * @category Conversion
 */

export const IntFromString = /*#__PURE__*/pipe( /*#__PURE__*/Pattern(intFromString, ['IntString', /*#__PURE__*/PrimitivesTypeString.int()[1]]), /*#__PURE__*/Brand(), /*#__PURE__*/Imap( /*#__PURE__*/PrimitivesGuard.int(), s => Number(s), n => n.toString()));
//# sourceMappingURL=IntFromString.mjs.map