/** @since 1.0.0 */
import { pipe } from "fp-ts/es6/function.js";
import * as O from "fp-ts/es6/Option.js";
import { deriveTypeString } from 'schemata-ts/derivations/type-string-schemable';
import { deriveGuard } from 'schemata-ts/Guard';
import { makeImplicitOptionalType } from 'schemata-ts/internal/struct';
import { Imap } from 'schemata-ts/schemata/Imap';
import { Nullable } from 'schemata-ts/schemata/Nullable';
import { Option } from 'schemata-ts/schemata/Option';
/**
 * Represents an optional type which decodes from null and undefined and encodes to null
 *
 * @since 1.0.0
 * @category Conversion
 */

export const OptionFromNullable = inner => {
  const guard = deriveGuard(Option(inner));
  return makeImplicitOptionalType(pipe(Nullable(inner), Imap({
    is: u => guard.is(u) && pipe(u, O.fold(() => true, a => a !== undefined && a !== null))
  }, O.fromNullable, O.toNullable, `Option<${deriveTypeString(inner)[1]}>`)));
};
//# sourceMappingURL=OptionFromNullable.mjs.map