/** @since 2.2.0 */
import * as E from "fp-ts/es6/Either.js";
import { flow, pipe } from "fp-ts/es6/function.js";
import * as Str from "fp-ts/es6/string.js";
import { getNumberBoundsInt, isFloat } from 'schemata-ts/schemables/primitives/utils';
import { Float } from 'schemata-ts/schemata/Float';
import { Parse } from 'schemata-ts/schemata/Parse';
/**
 * Parses a string into an float. A slightly more performant variant of the now deprecated
 * `S.FloatFromString` with the additional ability to specify `min` / `max` bounds.
 *
 * @since 2.2.0
 * @category Printer Parsers
 */

export const ParseFloat = params => pipe(Float(params), Parse(`FloatString${getNumberBoundsInt(params)}`, flow(Str.trim, E.fromPredicate(s => s.length > 0, () => 'Expected a non-empty float string'), E.map(Number), E.filterOrElse(isFloat(params), n => `Expected a floating point number between ${params?.min ?? Number.MIN_SAFE_INTEGER} and ${params?.max ?? Number.MAX_SAFE_INTEGER}, but got ${n}`)), flow(String, _ => _, E.right)));
//# sourceMappingURL=ParseFloat.mjs.map