/** @since 2.2.0 */
import * as E from "fp-ts/es6/Either.js";
import { flow, pipe } from "fp-ts/es6/function.js";
import * as Str from "fp-ts/es6/string.js";
import { getNumberBoundsInt, isInt } from 'schemata-ts/schemables/primitives/utils';
import { Int } from 'schemata-ts/schemata/Int';
import { Parse } from 'schemata-ts/schemata/Parse';
/**
 * Parses a string into an integer. A slightly more performant variant of the now
 * deprecated `S.IntFromString` with the additional ability to specify `min` / `max` bounds.
 *
 * @since 2.2.0
 * @category Printer Parsers
 */

export const ParseInt = params => pipe(Int(params), Parse(`IntString${getNumberBoundsInt(params)}`, flow(Str.trim, E.fromPredicate(s => s.length > 0, () => 'Expected a non-empty integer string'), E.map(Number), E.filterOrElse(isInt(params), n => `Expected an integer between ${params?.min ?? Number.MIN_SAFE_INTEGER} and ${params?.max ?? Number.MAX_SAFE_INTEGER}, but got ${n}`)), flow(String, _ => _, E.right)));
//# sourceMappingURL=ParseInt.mjs.map