/** @since 1.0.0 */
import { unsafeCoerce } from "fp-ts/es6/function.js";
import * as Sg from "fp-ts/es6/Semigroup.js";
import { deriveGuard } from 'schemata-ts/derivations/guard-schemable';
import { deriveInformation } from 'schemata-ts/derivations/information-schemable';
import { deriveTypeString } from 'schemata-ts/derivations/type-string-schemable';
import { make } from 'schemata-ts/internal/schema';
import { StructTypeString } from 'schemata-ts/schemables/struct/instances/type-string';
import { Optional } from 'schemata-ts/schemata/Optional';
/**
 * Used to construct a struct schema with enumerated keys where any number of known keys
 * are permitted.
 *
 * @deprecated Use `Struct({}).partial()` instead
 * @since 1.0.0
 * @category Combinators
 */

export const Partial = (props, extraProps = 'strip') => unsafeCoerce(make(_ => {
  const struct = {};
  const structName = {};

  for (const key in props) {
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    const schema = Optional(props[key]);
    const schemable = schema.runSchema(_);
    const guard = deriveGuard(schema);
    const information = deriveInformation(schema);
    const name = deriveTypeString(schema);
    struct[key] = {
      schemable,
      guard,
      information,
      semigroup: Sg.last(),
      name: name[0]
    };
    structName[key] = {
      schemable: name,
      guard,
      information,
      semigroup: Sg.last(),
      name: name[0]
    };
  }

  const wholeName = StructTypeString.struct(structName, extraProps, '');
  return _.struct(struct, extraProps, wholeName[0]);
}));
//# sourceMappingURL=Partial.mjs.map