/** @since 1.0.0 */
import { unsafeCoerce } from "fp-ts/es6/function.js";
import { SchemaImplementation } from 'schemata-ts/Schema';
/**
 * Represents string inputs / outputs
 *
 * @since 1.0.0
 * @category String
 */

export const String = params => new StringSchema(params);
/**
 * The StringSchema transformer class, use instead `S.String` function to create a StringSchema
 *
 * @since 2.2.0
 * @category Transformations
 */

export class StringSchema extends SchemaImplementation {
  params;

  constructor(params) {
    super(s => s.string(params));
    this.params = params;
  }
  /**
   * Brands this string schema with a certain brand
   *
   * @since 2.2.0
   */


  brand = () => unsafeCoerce(this);
  /**
   * Sets the minimum required length of the string
   *
   * @since 2.2.0
   */

  minLength = minLength => new StringSchema({ ...this.params,
    minLength
  });
  /**
   * Sets the maximum required length of the string
   *
   * @since 2.2.0
   */

  maxLength = maxLength => new StringSchema({ ...this.params,
    maxLength
  });
  /**
   * Overrides the 'expected' field in TranscodeError > TypeMismatch
   *
   * @since 2.2.0
   * @default 'string<minLength?, maxLength?>'
   */

  errorName = errorName => new StringSchema({ ...this.params,
    errorName
  });
}
//# sourceMappingURL=String.mjs.map