/** @since 1.4.0 */
import { deriveTypeString } from 'schemata-ts/derivations/type-string-schemable';
import { SchemaImplementation } from 'schemata-ts/Schema';
import { ArrayTypeString } from 'schemata-ts/schemables/array/instances/type-string';
/**
 * A schema for n-tuples
 *
 * @since 1.0.0
 * @category Combinators
 */

export const Tuple = (...items) => new TupleSchema(items);
/**
 * The TupleSchema transformer class, use instead `Tuple` function
 *
 * @since 2.2.0
 * @category Transformations
 */

export class TupleSchema extends SchemaImplementation {
  items;

  constructor(items) {
    const name = ArrayTypeString.tuple('', ...items.map(deriveTypeString))[0];
    super(s => s.tuple(name, ...items.map(schema => schema.runSchema(s))));
    this.items = items;
  }
  /**
   * Appends items to the end of the tuple
   *
   * @since 2.2.0
   */


  append = (...items) => new TupleSchema([...this.items, ...items]);
  /**
   * Prepends items to the beginning of the tuple
   *
   * @since 2.2.0
   */

  prepend = (...items) => new TupleSchema([...items, ...this.items]);
}
//# sourceMappingURL=Tuple.mjs.map