/** @since 1.0.0 */
import { pipe } from "fp-ts/es6/function.js";
import * as k from 'kuvio';
import * as Nt from 'schemata-ts/newtype';
import { Newtype } from 'schemata-ts/schemata/Newtype';
import { Pattern } from 'schemata-ts/schemata/Pattern';
/**
 * See: https://github.com/validatorjs/validator.js/blob/master/src/lib/isUUID.js
 *
 * @since 1.0.0
 * @category Pattern
 */

const uuidPattern = {
  1: k.patterns.uuidV1,
  2: k.patterns.uuidV2,
  3: k.patterns.uuidV3,
  4: k.patterns.uuidV4,
  5: k.patterns.uuidV5,
  any: k.patterns.anyUUID
};
/**
 * A newtype iso for UUID
 *
 * @since 2.0.0
 */

export const isoUUID = () => Nt.iso();
/**
 * @since 1.0.0
 * @category String
 */

export const UUID = version => pipe(Pattern(uuidPattern[version], `UUID version ${version}`), Newtype(isoUUID(), `UUID version ${version}`));
//# sourceMappingURL=UUID.mjs.map