/** @since 2.0.0 */
import { pipe, unsafeCoerce } from "fp-ts/es6/function.js";
import * as RNEA from "fp-ts/es6/ReadonlyNonEmptyArray.js";
import { deriveGuard } from 'schemata-ts/derivations/guard-schemable';
import { deriveInformation } from 'schemata-ts/derivations/information-schemable';
import { deriveTypeString } from 'schemata-ts/derivations/type-string-schemable';
import { make } from 'schemata-ts/internal/schema';
/**
 * Represents a union of different types. **Note: Union items ought to be mutually
 * exclusive**. To mitigate this, union items are first sorted from most complex to least
 * complex, and in the case of transcoder and guard the first match is kept.
 *
 * @since 2.0.0
 * @category Combinators
 */

export const Union = (...members) => {
  return unsafeCoerce(make(s => pipe(members, RNEA.map(schema => ({
    guard: deriveGuard(schema),
    precedence: deriveInformation(schema),
    name: deriveTypeString(schema)[0],
    member: schema.runSchema(s)
  })), guardedUnionMembers => s.guardedUnion(...guardedUnionMembers))));
};
//# sourceMappingURL=Union.mjs.map