"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.camelCase = void 0;

var _ChainRec = /*#__PURE__*/require("fp-ts/lib/ChainRec.js");

var E = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/Either.js"));

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

var O = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/Option.js"));

var Pred = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/Predicate.js"));

var RA = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/ReadonlyArray.js"));

var Str = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/string.js"));

var k = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("kuvio"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Parses a string and turns it into camelCase.
 *
 * @since 1.4.0
 */

/** Convert a character code to character */
const cc = String.fromCharCode;
/** A list of whitespace + "-" + "_" patterns to match type-fest's type-level camelcase */

const delimiters = /*#__PURE__*/k.characterClass(false, /*#__PURE__*/cc(9),
/*#__PURE__*/
// tab
cc(10),
/*#__PURE__*/
// line feed
cc(11),
/*#__PURE__*/
// vertical tab
cc(12),
/*#__PURE__*/
// form feed
cc(13),
/*#__PURE__*/
// carriage return
cc(32),
/*#__PURE__*/
// space
cc(133),
/*#__PURE__*/
// next line
cc(160),
/*#__PURE__*/
// non-breaking space
cc(5760),
/*#__PURE__*/
// ogham space mark
cc(8192),
/*#__PURE__*/
// en quad
cc(8193),
/*#__PURE__*/
// em quad
cc(8194),
/*#__PURE__*/
// en space
cc(8195),
/*#__PURE__*/
// em space
cc(8196),
/*#__PURE__*/
// three-per-em space
cc(8197),
/*#__PURE__*/
// four-per-em space
cc(8198),
/*#__PURE__*/
// six-per-em space
cc(8199),
/*#__PURE__*/
// figure space
cc(8200),
/*#__PURE__*/
// punctuation space
cc(8201),
/*#__PURE__*/
// thin space
cc(8202),
/*#__PURE__*/
// hair space
cc(8232),
/*#__PURE__*/
// line separator
cc(8233),
/*#__PURE__*/
// paragraph separator
cc(8239),
/*#__PURE__*/
// narrow no-break space
cc(8287),
/*#__PURE__*/
// medium mathematical space
cc(12288),
/*#__PURE__*/
// ideographic space
cc(65279), // zero width no-break space
'-', // hyphen-minus
'_');
const delimiterRegex = /*#__PURE__*/k.regexFromPattern(delimiters);

const dropRightWhile = predicate => as => (0, _function.pipe)(as, RA.findLastIndex(Pred.not(predicate)), O.fold(() => [], i => as.slice(0, i + 1)));

const trimDelimiters = /*#__PURE__*/(0, _function.flow)(Str.trim, /*#__PURE__*/Str.split(''), /*#__PURE__*/RA.dropLeftWhile(s => delimiterRegex.test(s)), /*#__PURE__*/dropRightWhile(s => delimiterRegex.test(s)), /*#__PURE__*/RA.foldMap(Str.Monoid)(_function.identity));

const isLetter = s => /[a-zA-Z]/.test(s);

const isUppercase = s => isLetter(s) && s === s.toUpperCase();

const isNumber = char => /[0-9]/.test(char);

const hasLowercase = s => /[a-z]/gm.test(s);
/**
 * Parses a string and turns it into camelCase.
 *
 * @since 1.4.0
 */


const camelCase = input => {
  if (input.length === 0) return input;

  const go = s => {
    const char = s.input.charAt(s.index);
    const nextChar = (0, _function.pipe)(s.input.charAt(s.index + 1), O.fromPredicate(s => s.length > 0)); // Stream is exhausted

    if (char === '') {
      return E.right(s.output);
    } // Lowercase the first char


    if (s.index === 0) {
      return E.left({
        input: s.input,
        output: s.output + char.toLowerCase(),
        index: s.index + 1,
        lastCharWasDelimiter: false,
        lastCharWasNumber: isNumber(char),
        hasBeenScreaming: isUppercase(char)
      });
    } // If the character is a delimiter, skip it


    if (delimiterRegex.test(char)) {
      return E.left({
        input: s.input,
        output: s.output,
        index: s.index + 1,
        lastCharWasDelimiter: true,
        lastCharWasNumber: false,
        hasBeenScreaming: false
      });
    } // If the last char was screaming, lowercase the current char if the next char is uppercase


    if (s.hasBeenScreaming && isUppercase(char)) {
      return E.left({
        input: s.input,
        output: s.output + (O.isSome(nextChar) && isUppercase(nextChar.value) ? char.toLowerCase() : char),
        index: s.index + 1,
        lastCharWasDelimiter: false,
        lastCharWasNumber: isNumber(char),
        hasBeenScreaming: true
      });
    } // If the last char was a delimiter (or number), uppercase the current char


    if (s.lastCharWasDelimiter || s.lastCharWasNumber) {
      return E.left({
        input: s.input,
        output: s.output + char.toUpperCase(),
        index: s.index + 1,
        hasBeenScreaming: isUppercase(char),
        lastCharWasNumber: isNumber(char),
        lastCharWasDelimiter: false
      });
    }

    return E.left({
      input: s.input,
      output: s.output + char,
      index: s.index + 1,
      lastCharWasDelimiter: false,
      lastCharWasNumber: isNumber(char),
      hasBeenScreaming: isUppercase(char)
    });
  };

  const trimmed = trimDelimiters(input);
  if (trimmed.length === 0) return trimmed;
  return (0, _ChainRec.tailRec)({
    input: hasLowercase(trimmed) ? trimmed : trimmed.toLowerCase(),
    output: '',
    index: 0,
    lastCharWasDelimiter: false,
    lastCharWasNumber: false,
    hasBeenScreaming: isUppercase(trimmed.charAt(0))
  }, go);
};

exports.camelCase = camelCase;
//# sourceMappingURL=camelcase.js.map