import { type Const } from 'fp-ts/Const';
import * as RNEA from 'fp-ts/ReadonlyNonEmptyArray';
import * as RR from 'fp-ts/ReadonlyRecord';
import type * as hkt from 'schemata-ts/internal/schemable';
export type JsonSchemaValue = JsonEmpty | JsonString | JsonNumber | JsonBoolean | JsonNull | JsonInteger | JsonConst | JsonLiteral | JsonStruct | JsonArray | JsonUnion | JsonIntersection | JsonRef;
export type JsonSchema = JsonSchemaValue & Description & References & Default;
export interface Description {
    readonly title?: string;
    readonly description?: string;
    readonly readOnly?: boolean;
    readonly deprecated?: boolean;
}
export interface References {
    readonly $defs?: RR.ReadonlyRecord<string, JsonSchema>;
}
export interface Default {
    readonly default?: unknown;
}
export declare const make: <A>(value: JsonSchemaValue) => Const<JsonSchema, A>;
/** Matches anything */
export declare class JsonEmpty {
}
/** Matches a subset of strings */
export declare class JsonString {
    readonly minLength?: number | undefined;
    readonly maxLength?: number | undefined;
    readonly pattern?: string | undefined;
    readonly contentEncoding?: string | undefined;
    readonly contentMediaType?: string | undefined;
    readonly contentSchema?: JsonSchema | undefined;
    readonly format?: string | undefined;
    readonly type = "string";
    constructor(minLength?: number | undefined, maxLength?: number | undefined, pattern?: string | undefined, contentEncoding?: string | undefined, contentMediaType?: string | undefined, contentSchema?: JsonSchema | undefined, format?: string | undefined);
}
/** Matches a subset of floats */
export declare class JsonNumber {
    readonly minimum?: number | undefined;
    readonly maximum?: number | undefined;
    readonly type = "number";
    constructor(minimum?: number | undefined, maximum?: number | undefined);
}
/** Matches a subset of integers */
export declare class JsonInteger implements Omit<JsonNumber, 'type'> {
    readonly minimum?: number | undefined;
    readonly maximum?: number | undefined;
    readonly type = "integer";
    constructor(minimum?: number | undefined, maximum?: number | undefined);
}
/** Matches true or false */
export declare class JsonBoolean {
    readonly type = "boolean";
}
/** Matches a constant value */
export declare class JsonConst {
    readonly const: unknown;
}
/** Matches a boolean, number, string, or null constant value */
export type JsonLiteral = (JsonBoolean | JsonNumber | JsonString | JsonNull) & JsonConst;
/** Matches a set of properties with a given set of required properties. */
export declare class JsonStruct {
    readonly properties: Readonly<Record<string, JsonSchema>>;
    readonly required: ReadonlyArray<string>;
    readonly additionalProperties?: false | JsonSchema | undefined;
    readonly propertyNames?: JsonSchema | undefined;
    readonly type = "object";
    constructor(properties: Readonly<Record<string, JsonSchema>>, required: ReadonlyArray<string>, additionalProperties?: false | JsonSchema | undefined, propertyNames?: JsonSchema | undefined);
}
/** Matches a subset of arrays with uniform index values (or specific index values) */
export declare class JsonArray {
    readonly items: JsonSchema | ReadonlyArray<JsonSchema>;
    readonly minItems?: number | undefined;
    readonly maxItems?: number | undefined;
    readonly type = "array";
    constructor(items: JsonSchema | ReadonlyArray<JsonSchema>, minItems?: number | undefined, maxItems?: number | undefined);
}
/** Matches null exactly */
export declare class JsonNull {
    readonly type = "null";
    readonly const: null;
}
/** Matches any of the supplied schemas */
export declare class JsonUnion {
    readonly anyOf: ReadonlyArray<JsonSchema>;
    constructor(anyOf: ReadonlyArray<JsonSchema>);
}
/** Matches all of the supplied schemas */
export declare class JsonIntersection {
    readonly allOf: RNEA.ReadonlyNonEmptyArray<JsonSchema>;
    constructor(allOf: RNEA.ReadonlyNonEmptyArray<JsonSchema>);
}
/**
 * A reference to a named schema definition
 *
 * @since 2.0.0
 */
export declare class JsonRef {
    readonly $ref: string;
    constructor($ref: string);
}
export interface SchemableLambda extends hkt.SchemableLambda {
    readonly type: Const<JsonSchema, this['Input']>;
}
//# sourceMappingURL=json-schema.d.ts.map