"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.stripIdentity = exports.make = exports.asDraft07 = exports.as2020 = exports.addReferences = exports.addDescription = exports.addDefault = exports.JsonUnion = exports.JsonStruct = exports.JsonString = exports.JsonRef = exports.JsonNumber = exports.JsonNull = exports.JsonIntersection = exports.JsonInteger = exports.JsonEmpty = exports.JsonConst = exports.JsonBoolean = exports.JsonArray = void 0;

var _Const = /*#__PURE__*/require("fp-ts/lib/Const.js");

var E = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/Either.js"));

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

var RA = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/ReadonlyArray.js"));

var RNEA = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/ReadonlyNonEmptyArray.js"));

var RR = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/ReadonlyRecord.js"));

var _util = /*#__PURE__*/require("schemata-ts/internal/util");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const make = _Const.make;
/** @internal */

exports.make = make;

const addDescription = description => schema => (0, _Const.make)(Object.assign({}, schema, description));
/** @internal */


exports.addDescription = addDescription;

const addReferences = references => schema => (0, _Const.make)(Object.assign({}, schema, references));
/** @internal */


exports.addReferences = addReferences;

const addDefault = defaultValue => schema => (0, _Const.make)(Object.assign({}, schema, {
  default: defaultValue
}));
/** Matches anything */


exports.addDefault = addDefault;

class JsonEmpty {}
/** Matches a subset of strings */


exports.JsonEmpty = JsonEmpty;

class JsonString {
  minLength;
  maxLength;
  pattern;
  contentEncoding;
  contentMediaType;
  contentSchema;
  format;
  type = 'string';

  constructor(minLength, maxLength, pattern, contentEncoding, contentMediaType, contentSchema, format) {
    this.minLength = minLength;
    this.maxLength = maxLength;
    this.pattern = pattern;
    this.contentEncoding = contentEncoding;
    this.contentMediaType = contentMediaType;
    this.contentSchema = contentSchema;
    this.format = format;
  }

}
/** Matches a subset of floats */


exports.JsonString = JsonString;

class JsonNumber {
  minimum;
  maximum;
  type = 'number';

  constructor(minimum, maximum) {
    this.minimum = minimum;
    this.maximum = maximum;
  }

}
/** Matches a subset of integers */


exports.JsonNumber = JsonNumber;

class JsonInteger {
  minimum;
  maximum;
  type = 'integer';

  constructor(minimum, maximum) {
    this.minimum = minimum;
    this.maximum = maximum;
  }

}
/** Matches true or false */


exports.JsonInteger = JsonInteger;

class JsonBoolean {
  type = 'boolean';
}
/** Matches a constant value */


exports.JsonBoolean = JsonBoolean;

class JsonConst {
  const;
}
/** Matches a set of properties with a given set of required properties. */


exports.JsonConst = JsonConst;

class JsonStruct {
  properties;
  required;
  additionalProperties;
  propertyNames;
  type = 'object';

  constructor(properties, required, additionalProperties, propertyNames) {
    this.properties = properties;
    this.required = required;
    this.additionalProperties = additionalProperties;
    this.propertyNames = propertyNames;
  }

}
/** Matches a subset of arrays with uniform index values (or specific index values) */


exports.JsonStruct = JsonStruct;

class JsonArray {
  items;
  minItems;
  maxItems;
  type = 'array';

  constructor(items, minItems, maxItems) {
    this.items = items;
    this.minItems = minItems;
    this.maxItems = maxItems;
  }

}
/** Matches null exactly */


exports.JsonArray = JsonArray;

class JsonNull {
  type = 'null';
  const = null;
}
/** Matches any of the supplied schemas */


exports.JsonNull = JsonNull;

class JsonUnion {
  anyOf;

  constructor(anyOf) {
    this.anyOf = anyOf;
  }

}
/** Matches all of the supplied schemas */


exports.JsonUnion = JsonUnion;

class JsonIntersection {
  allOf;

  constructor(allOf) {
    this.allOf = allOf;
  }

}
/**
 * A reference to a named schema definition
 *
 * @since 2.0.0
 */


exports.JsonIntersection = JsonIntersection;

class JsonRef {
  $ref;

  constructor($ref) {
    this.$ref = $ref;
  }

}
/** @internal */


exports.JsonRef = JsonRef;

const stripIdentity = schema => JSON.parse(JSON.stringify(schema));
/** @internal */
// istanbul ignore next


exports.stripIdentity = stripIdentity;

class References_ {
  $defs;

  constructor($defs) {
    this.$defs = $defs;
  }

}
/** A recursive key setter */


const recurseKeyOf = (_, key, mapper) => {
  return method => schema => {
    if (!(0, _util.hasOwn)(schema, key) || schema[key] === undefined) {
      return schema;
    }

    const {
      [key]: value,
      ...rest
    } = schema;
    const result = mapper(method)(value);

    if (E.isRight(result)) {
      const [newKey, _, inject] = result.right;
      return Object.assign({}, rest, {
        [newKey]: _
      }, inject);
    }

    const [_, inject] = result.left;
    return Object.assign({}, rest, {
      [key]: _
    }, inject);
  };
};

const foldDefault = method => schema => method(schema);

const foldDefaultLeft = method => (0, _function.flow)(foldDefault(method), _function.tuple, E.left); // -----------------
// Json Remappers
// -----------------


const remapContentSchema = /*#__PURE__*/recurseKeyOf(JsonString, 'contentSchema', foldDefaultLeft);
const remapProperties = /*#__PURE__*/recurseKeyOf(JsonStruct, 'properties', method => (0, _function.flow)(RR.map(method), _function.tuple, E.left));
const remapAdditionalProperties = /*#__PURE__*/recurseKeyOf(JsonStruct, 'additionalProperties', foldDefaultLeft);
const remapPropertyNames = /*#__PURE__*/recurseKeyOf(JsonStruct, 'propertyNames', foldDefaultLeft);
const remapItems = /*#__PURE__*/recurseKeyOf(JsonArray, 'items', method => items => E.left((0, _function.tuple)(Array.isArray(items) ? items.map(method) : method(items))));
const remapAnyOf = /*#__PURE__*/recurseKeyOf(JsonUnion, 'anyOf', _ => (0, _function.flow)(RA.map(_), _function.tuple, E.left));
const remapAllOf = /*#__PURE__*/recurseKeyOf(JsonIntersection, 'allOf', _ => (0, _function.flow)(RNEA.map(_), _function.tuple, E.left));
const remapDefs = /*#__PURE__*/recurseKeyOf(References_, '$defs', _ => (0, _function.flow)(RR.map(_), _function.tuple, E.left)); // ---------------------
// Json Schema Draft-07
// ---------------------

/** @internal */

const asDraft07 = schema => (0, _function.pipe)(schema, defsToDefinitions(asDraft07), recurseDraft07);

exports.asDraft07 = asDraft07;
const defsToDefinitions = /*#__PURE__*/recurseKeyOf(References_, '$defs', () => (0, _function.flow)(RR.map(asDraft07), _ => E.right((0, _function.tuple)('definitions', _))));
const recurseDraft07 = /*#__PURE__*/(0, _function.flow)( /*#__PURE__*/remapContentSchema(asDraft07), /*#__PURE__*/remapProperties(asDraft07), /*#__PURE__*/remapAdditionalProperties(asDraft07), /*#__PURE__*/remapPropertyNames(asDraft07), /*#__PURE__*/remapItems(asDraft07), /*#__PURE__*/remapAnyOf(asDraft07), /*#__PURE__*/remapAllOf(asDraft07)); // ---------------------
// Json Schema 2020-12
// ---------------------

/** @internal */

const as2020 = schema => (0, _function.pipe)(schema, itemsToPrefixItems(as2020), recurse2020);

exports.as2020 = as2020;
const itemsToPrefixItems = /*#__PURE__*/recurseKeyOf(JsonArray, 'items', () => schema => Array.isArray(schema) ? E.right((0, _function.tuple)('prefixItems', schema.map(as2020), {
  items: false
})) : E.left((0, _function.tuple)(as2020(schema))));
const recurse2020 = /*#__PURE__*/(0, _function.flow)( /*#__PURE__*/remapDefs(as2020), /*#__PURE__*/remapContentSchema(as2020), /*#__PURE__*/remapProperties(as2020), /*#__PURE__*/remapAdditionalProperties(as2020), /*#__PURE__*/remapPropertyNames(as2020), /*#__PURE__*/remapAnyOf(as2020), /*#__PURE__*/remapAllOf(as2020));
//# sourceMappingURL=json-schema.js.map