"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isBeforeInc = exports.isAfterInc = exports.getDateBoundsStr = exports.earliestSafeDate = void 0;
exports.isSafeDate = isSafeDate;
exports.smallestSafeMs = exports.latestSafeDate = exports.largestSafeMs = exports.isValidDateString = void 0;

/** @internal */
function isSafeDate(d) {
  return d instanceof Date && !Number.isNaN(d.getTime());
}
/** @internal */


const largestSafeMs = 8640000000000000;
/** @internal */

exports.largestSafeMs = largestSafeMs;
const smallestSafeMs = -8640000000000000;
/** @internal */

exports.smallestSafeMs = smallestSafeMs;

const isValidDateString = s => typeof s === 'string' && !Number.isNaN(new Date(s).getTime());
/** @internal */


exports.isValidDateString = isValidDateString;

const isBeforeInc = d => d2 => d.getTime() <= d2.getTime();
/** @internal */


exports.isBeforeInc = isBeforeInc;

const isAfterInc = d => d2 => d.getTime() >= d2.getTime();
/** @internal */


exports.isAfterInc = isAfterInc;
const earliestSafeDate = /*#__PURE__*/new Date(smallestSafeMs);
/** @internal */

exports.earliestSafeDate = earliestSafeDate;
const latestSafeDate = /*#__PURE__*/new Date(largestSafeMs);
exports.latestSafeDate = latestSafeDate;

const getDateBoundsStr = ( // istanbul ignore next
params = {}) => {
  const {
    afterDate,
    beforeDate
  } = params;
  if (afterDate === undefined && beforeDate === undefined) return ''; // istanbul ignore next

  const beforeDateStr = beforeDate?.toISOString(); // istanbul ignore next

  const afterDateStr = afterDate?.toISOString();
  return `<${// istanbul ignore next
  afterDateStr ?? ''},${// istanbul ignore next
  beforeDateStr ?? ''}>`;
};

exports.getDateBoundsStr = getDateBoundsStr;
//# sourceMappingURL=utils.js.map