"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isInt = exports.isFloat = exports.getNumberBoundsInt = exports.getLengthBoundsString = void 0;

const isInt = (params = {}) => n => {
  const {
    min = Number.MIN_SAFE_INTEGER,
    max = Number.MAX_SAFE_INTEGER
  } = params;
  return typeof n === 'number' && !Number.isNaN(n) && Number.isSafeInteger(n) && n >= min && n <= max;
};

exports.isInt = isInt;

const isFloat = (params = {}) => n => {
  const {
    min = -Number.MAX_VALUE,
    max = Number.MAX_VALUE
  } = params;
  return typeof n === 'number' && !Number.isNaN(n) && n >= min && n <= max;
}; // istanbul ignore next


exports.isFloat = isFloat;

const getLengthBoundsString = (params = {}, lchar = '<', rchar = '>') => {
  const {
    minLength,
    maxLength
  } = params;
  if (minLength === undefined && maxLength === undefined) return '';
  return `${lchar}${minLength ?? ''},${maxLength ?? ''}${rchar}`;
};

exports.getLengthBoundsString = getLengthBoundsString;

const getNumberBoundsInt = ( // istanbul ignore next
params = {}) => {
  const {
    min,
    max
  } = params;
  if (min === undefined && max === undefined) return '';
  return `<${min ?? ''},${max ?? ''}>`;
};

exports.getNumberBoundsInt = getNumberBoundsInt;
//# sourceMappingURL=utils.js.map