"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.safeIntersect = exports.remapPropertyKeys = exports.getValidateObject = void 0;

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

var O = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/Option.js"));

var RA = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/ReadonlyArray.js"));

var RNEA = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/ReadonlyNonEmptyArray.js"));

var RR = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/ReadonlyRecord.js"));

var _struct = /*#__PURE__*/require("schemata-ts/internal/struct");

var TC = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("schemata-ts/internal/transcoder"));

var _util = /*#__PURE__*/require("schemata-ts/internal/util");

var _definition = /*#__PURE__*/require("schemata-ts/schemables/guarded-union/definition");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Collapses a struct definition into a record with remapped keys whose values are union
 * members of every output key collision for that key.
 */
const remapPropertyKeys = (properties, mapInformation = _function.identity) => {
  // --- a lookup table of output keys to union items
  const lookupByOutputKey_ = {};

  for (const key in properties) {
    // -- ignore inherited properties
    // istanbul ignore next
    if (!(0, _util.hasOwn)(properties, key)) continue; // eslint-disable-next-line @typescript-eslint/no-non-null-assertion

    const {
      schemable,
      guard,
      information,
      semigroup,
      name
    } = properties[key];
    const inputKey = key;
    const outputKey = (0, _function.pipe)((0, _struct.getKeyRemap)(schemable), O.getOrElse(() => inputKey));
    const nextUnionItem = {
      member: schemable,
      guard,
      precedence: mapInformation(information),
      inputKey,
      semigroup,
      name
    };
    const outputKeyUnion = (0, _function.pipe)(lookupByOutputKey_, RR.lookup(outputKey), O.fold(() => RNEA.of(nextUnionItem), RA.append(nextUnionItem)));
    lookupByOutputKey_[outputKey] = outputKeyUnion;
  }

  return (0, _function.pipe)(lookupByOutputKey_, RR.map(RNEA.sort(_definition.ordGuardedPrecedentedUnionMember)));
};

exports.remapPropertyKeys = remapPropertyKeys;

const safeIntersect = (a, b) => {
  const isObjectA = typeof a === 'object' && a !== null;
  const isObjectB = typeof b === 'object' && b !== null;
  if (isObjectA || isObjectB) return Object.assign({}, a, b);
  return void 0;
};

exports.safeIntersect = safeIntersect;

const getValidateObject = M => name => u => {
  if (u === null || typeof u !== 'object' || Array.isArray(u)) {
    return M.throwError(TC.transcodeErrors(TC.typeMismatch(name, u)));
  }

  return M.of(u);
};

exports.getValidateObject = getValidateObject;
//# sourceMappingURL=utils.js.map