"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Annotate = void 0;

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

var RR = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/ReadonlyRecord.js"));

var _jsonSchemaSchemable = /*#__PURE__*/require("schemata-ts/derivations/json-schema-schemable");

var _schema = /*#__PURE__*/require("schemata-ts/internal/schema");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/** @since 1.2.0 */

/**
 * Annotate a Json Schema with title, description, and references.
 *
 * Note: references must be specified for all "Lazy" schemas, and must occur after the
 * declaration of the primary schema. This is because mutual-recursion for json-schema is
 * implemented as a ref, and that ref must be specified in the parent using `Annotate`
 *
 * @since 1.2.0
 * @category Combinators
 */
const Annotate = params => schema => {
  const {
    references = {}
  } = params;
  const mappedRefs = (0, _function.pipe)(references, RR.map(_jsonSchemaSchemable.deriveJsonSchema));
  return (0, _schema.make)(s => s.annotate({ ...(params.title === undefined ? {} : {
      title: params.title
    }),
    ...(params.description === undefined ? {} : {
      description: params.description
    }),
    ...(params.references === undefined ? {} : {
      references: mappedRefs
    }),
    ...(params.typeString === undefined ? {} : {
      typeString: params.typeString
    }),
    ...(params.readOnly === undefined ? {} : {
      readOnly: params.readOnly
    }),
    ...(params.deprecated === undefined ? {} : {
      deprecated: params.deprecated
    })
  })(schema.runSchema(s)));
};

exports.Annotate = Annotate;
//# sourceMappingURL=Annotate.js.map