import type * as RNEA from 'fp-ts/ReadonlyNonEmptyArray';
import { type Schema, SchemaImplementation } from 'schemata-ts/Schema';
type ArrayParams = {
    readonly minLength?: number;
    readonly maxLength?: number;
    readonly errorName?: string;
};
/**
 * An array type of known values.
 *
 * @since 1.0.0
 * @category Combinators
 */
export declare const Array: <I, O>(codomain: Schema<I, O>, params?: ArrayParams) => ArraySchema<I, O>;
/**
 * The ArraySchema schema transformer, to construct an ArraySchema use `S.Array` combinator instead.
 *
 * @since 2.2.0
 * @category Transformations
 */
export declare class ArraySchema<I, O> extends SchemaImplementation<ReadonlyArray<I>, ReadonlyArray<O>> {
    private readonly codomain;
    private readonly params;
    constructor(codomain: Schema<I, O>, params?: ArrayParams);
    /**
     * Requires a minimum array length
     *
     * Returns a new ArraySchema
     *
     * @since 2.2.0
     */
    readonly minLength: (minLength: number) => ArraySchema<I, O>;
    /**
     * Requires a maximum array length
     *
     * Returns a new ArraySchema
     *
     * @since 2.2.0
     */
    readonly maxLength: (maxLength: number) => ArraySchema<I, O>;
    /**
     * Converts an array schema to a non-empty array schema
     *
     * Returns a new Schema
     *
     * @since 2.2.0
     */
    readonly nonEmpty: () => Schema<RNEA.ReadonlyNonEmptyArray<I>, RNEA.ReadonlyNonEmptyArray<O>>;
}
export {};
//# sourceMappingURL=Array.d.ts.map