"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ArraySchema = exports.Array = void 0;

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

var _typeStringSchemable = /*#__PURE__*/require("schemata-ts/derivations/type-string-schemable");

var _Schema = /*#__PURE__*/require("schemata-ts/Schema");

var _typeString = /*#__PURE__*/require("schemata-ts/schemables/array/instances/type-string");

/** @since 1.4.0 */

/**
 * An array type of known values.
 *
 * @since 1.0.0
 * @category Combinators
 */
const Array = (codomain, params) => new ArraySchema(codomain, params);
/**
 * The ArraySchema schema transformer, to construct an ArraySchema use `S.Array` combinator instead.
 *
 * @since 2.2.0
 * @category Transformations
 */


exports.Array = Array;

class ArraySchema extends _Schema.SchemaImplementation {
  codomain;
  params;

  constructor(codomain, params = {}) {
    const {
      errorName: errorName_
    } = params;
    const expectedName = errorName_ ?? (0, _function.pipe)((0, _typeStringSchemable.deriveTypeString)(codomain), _typeString.ArrayTypeString.array({ ...params,
      expectedName: ''
    }))[0];
    super(_ => _.array({ ...params,
      expectedName
    })(codomain.runSchema(_)));
    this.codomain = codomain;
    this.params = params;
  }
  /**
   * Requires a minimum array length
   *
   * Returns a new ArraySchema
   *
   * @since 2.2.0
   */


  minLength = minLength => new ArraySchema(this.codomain, { ...this.params,
    minLength
  });
  /**
   * Requires a maximum array length
   *
   * Returns a new ArraySchema
   *
   * @since 2.2.0
   */

  maxLength = maxLength => new ArraySchema(this.codomain, { ...this.params,
    maxLength
  });
  /**
   * Converts an array schema to a non-empty array schema
   *
   * Returns a new Schema
   *
   * @since 2.2.0
   */

  nonEmpty = () => (0, _function.unsafeCoerce)(this.minLength(1));
}

exports.ArraySchema = ArraySchema;
//# sourceMappingURL=Array.js.map