import * as k from 'kuvio';
import { type Branded } from 'schemata-ts/brand';
import { type Schema } from 'schemata-ts/Schema';
type BigIntStringBrand = {
    readonly BigIntString: unique symbol;
};
/**
 * Represents strings that can be parsed properly by `BigInt()`
 *
 * @since 2.0.0
 */
export type BigIntString = Branded<string, BigIntStringBrand>;
/**
 * Controls the output base of the encoded string. Currently only accepts 2, 8, 10, and
 * 16. It does not decode in this specified base, and accepts any base as input: 2, 8, 10, or 16.
 *
 * @since 1.0.0
 */
export type BigIntFromStringParams = {
    readonly encodeToBase?: 2 | 8 | 10 | 16;
};
/**
 * @since 1.0.0
 * @category Pattern
 */
export declare const bigIntString: k.Pattern;
/**
 * Represents bigints converted from strings
 *
 * @since 1.0.0
 * @category Conversion
 */
export declare const BigIntFromString: Schema<BigIntString, bigint>;
export {};
//# sourceMappingURL=BigIntFromString.d.ts.map