import { type OptionalInputProps, type RequiredInputProps } from 'schemata-ts/internal/schema';
import { type OutputOf, type Schema } from 'schemata-ts/Schema';
import type { CamelCase, Simplify } from 'type-fest';
/**
 * The same as the `Struct` schema combinator, but keys are transformed to camel case in
 * the output type.
 *
 * **Warning** It is possible to have one or more input keys map to the same output key.
 * This combination will produce unlawful instances.
 *
 * @since 1.4.0
 * @category Combinators
 */
export declare const CamelCaseKeys: <T extends Record<string, Schema<any, any>>>(props: T, extraProps?: 'strip' | 'error', mergeStrategy?: 'first' | 'last') => Schema<Simplify<RequiredInputProps<T> & OptionalInputProps<T>>, {
    [K in keyof T as CamelCase<K, {
        preserveConsecutiveUppercase: true;
    }>]: OutputOf<T[K]>;
}>;
//# sourceMappingURL=CamelCaseKeys.d.ts.map