"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CamelCaseKeys = void 0;

var _guardSchemable = /*#__PURE__*/require("schemata-ts/derivations/guard-schemable");

var _informationSchemable = /*#__PURE__*/require("schemata-ts/derivations/information-schemable");

var _mergeSemigroupSchemable = /*#__PURE__*/require("schemata-ts/derivations/merge-semigroup-schemable");

var _typeStringSchemable = /*#__PURE__*/require("schemata-ts/derivations/type-string-schemable");

var _camelcase = /*#__PURE__*/require("schemata-ts/internal/camelcase");

var _schema = /*#__PURE__*/require("schemata-ts/internal/schema");

var _struct = /*#__PURE__*/require("schemata-ts/internal/struct");

var _typeString = /*#__PURE__*/require("schemata-ts/schemables/struct/instances/type-string");

/** @since 1.4.0 */

/**
 * The same as the `Struct` schema combinator, but keys are transformed to camel case in
 * the output type.
 *
 * **Warning** It is possible to have one or more input keys map to the same output key.
 * This combination will produce unlawful instances.
 *
 * @since 1.4.0
 * @category Combinators
 */
const CamelCaseKeys = (props, extraProps = 'strip', mergeStrategy) => (0, _schema.make)(_ => {
  const struct = {};
  const structName = {};

  for (const key in props) {
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    const schema = props[key];
    const schemable = (0, _struct.remapKey)(schema.runSchema(_), _.clone, (0, _camelcase.camelCase)(key));
    const guard = (0, _guardSchemable.deriveGuard)(schema);
    const information = (0, _informationSchemable.deriveInformation)(schema);
    const name = (0, _typeStringSchemable.deriveTypeString)(schema);
    const semigroup = (0, _mergeSemigroupSchemable.deriveMergeSemigroup)(schema).semigroup(mergeStrategy);
    struct[key] = {
      schemable,
      guard,
      information,
      semigroup,
      name: name[0]
    };
    structName[key] = {
      schemable: name,
      guard,
      information,
      semigroup,
      name: name[0]
    };
  }

  const wholeName = _typeString.StructTypeString.struct(structName, extraProps, '');

  return _.struct(struct, extraProps, wholeName[0]);
});

exports.CamelCaseKeys = CamelCaseKeys;
//# sourceMappingURL=CamelCaseKeys.js.map