"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.minutesSeconds = exports.DateFromIsoString = void 0;

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

var k = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("kuvio"));

var _guardSchemable = /*#__PURE__*/require("schemata-ts/derivations/guard-schemable");

var _Brand = /*#__PURE__*/require("schemata-ts/schemata/Brand");

var _Date = /*#__PURE__*/require("schemata-ts/schemata/Date");

var _Imap = /*#__PURE__*/require("schemata-ts/schemata/Imap");

var _Pattern = /*#__PURE__*/require("schemata-ts/schemata/Pattern");

var _Refine = /*#__PURE__*/require("schemata-ts/schemata/Refine");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/** @since 1.0.0 */

/**
 * E.g. `2022`
 *
 * @since 1.0.0
 * @category Pattern
 */
const year = /*#__PURE__*/(0, _function.pipe)(k.digit, /*#__PURE__*/k.exactly(4));
/**
 * E.g. `+-002022`
 *
 * @since 1.0.0
 * @category Pattern
 */

const expandedYear = /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.characterClass(false, '+', '-'), k.subgroup, /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)(k.digit, /*#__PURE__*/k.exactly(6))));
/**
 * E.g. 01, 02, 03, 04, 05, 06, 07, 08, 09, 10, 11, 12
 *
 * @since 1.0.0
 * @category Pattern
 */

const month = /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('0'), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.integerRange(1, 9), k.subgroup)), k.subgroup, /*#__PURE__*/k.or( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.integerRange(10, 12), k.subgroup)), k.subgroup);
/**
 * E.g. 01, 02, 03, 04, 05, ..., 30, 31
 *
 * @since 1.0.0
 * @category Pattern
 */

const day = /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('0'), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.integerRange(1, 9), k.subgroup)), k.subgroup, /*#__PURE__*/k.or( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.integerRange(10, 31), k.subgroup)), k.subgroup);
/**
 * E.g. `YYYY-MM-DD`
 *
 * @since 1.0.0
 * @category Pattern
 */

const yearMonthDay = /*#__PURE__*/k.sequence( /*#__PURE__*/k.subgroup( /*#__PURE__*/k.oneOf(year, expandedYear)), /*#__PURE__*/k.char('-'), month, /*#__PURE__*/k.char('-'), day);
/**
 * E.g. `YYYY-MM`
 *
 * @since 1.0.0
 * @category Pattern
 */

const yearMonth = /*#__PURE__*/k.subgroup( /*#__PURE__*/k.sequence( /*#__PURE__*/k.subgroup( /*#__PURE__*/k.oneOf(year, expandedYear)), /*#__PURE__*/k.char('-'), month));
/**
 * E.G. `YYYY-MM-DD` or `YYYY-MM` or `YYYY`
 *
 * @since 1.0.0
 * @category Pattern
 */

const date = /*#__PURE__*/k.subgroup( /*#__PURE__*/k.oneOf(yearMonthDay, yearMonth, year, expandedYear));
/**
 * E.g. 00, 01, 02, 03, 04, 05, 06, 07, ..., 22, 23
 *
 * @since 1.0.0
 * @category Pattern
 */

const hour = /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('0'), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.integerRange(0, 9), k.subgroup)), k.subgroup, /*#__PURE__*/k.or( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.integerRange(10, 23), k.subgroup)), k.subgroup);
/**
 * E.g. 00, 01, 02, 03, 04, 05, 06, 07, ..., 58, 59
 *
 * @since 1.0.0
 * @category Pattern
 */

const minutesSeconds = /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('0'), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.integerRange(0, 9), k.subgroup)), k.subgroup, /*#__PURE__*/k.or( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.integerRange(10, 59), k.subgroup)), k.subgroup);
/**
 * @since 1.0.0
 * @category Pattern
 */

exports.minutesSeconds = minutesSeconds;
const milliseconds = /*#__PURE__*/(0, _function.pipe)(k.digit, /*#__PURE__*/k.atLeastOne(), k.subgroup);
/**
 * E.g. `Z` or `±05:00`
 *
 * @since 1.0.0
 * @category Pattern
 */

const timezoneOffset = /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('Z'), /*#__PURE__*/k.or( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.characterClass(false, '+', '-'), /*#__PURE__*/k.andThen(hour), /*#__PURE__*/k.andThen( /*#__PURE__*/k.char(':')), /*#__PURE__*/k.andThen(minutesSeconds))), k.subgroup);
/**
 * E.g. `HH:mm:ss.SSS`
 *
 * @since 1.0.0
 * @category Pattern
 */

const hrMinSecMs = /*#__PURE__*/(0, _function.pipe)(hour, /*#__PURE__*/k.andThen( /*#__PURE__*/k.char(':')), /*#__PURE__*/k.andThen(minutesSeconds), /*#__PURE__*/k.andThen( /*#__PURE__*/k.char(':')), /*#__PURE__*/k.andThen(minutesSeconds), /*#__PURE__*/k.andThen( /*#__PURE__*/k.char('.')), /*#__PURE__*/k.andThen(milliseconds));
/**
 * E.g. `HH:mm:ss`
 *
 * @since 1.0.0
 * @category Pattern
 */

const hrMinSec = /*#__PURE__*/(0, _function.pipe)(hour, /*#__PURE__*/k.andThen( /*#__PURE__*/k.char(':')), /*#__PURE__*/k.andThen(minutesSeconds), /*#__PURE__*/k.andThen( /*#__PURE__*/k.char(':')), /*#__PURE__*/k.andThen(minutesSeconds));
/**
 * E.g. `HH:mm`
 *
 * @since 1.0.0
 * @category Pattern
 */

const hrMin = /*#__PURE__*/(0, _function.pipe)(hour, /*#__PURE__*/k.andThen( /*#__PURE__*/k.char(':')), /*#__PURE__*/k.andThen(minutesSeconds));
/**
 * Iso time string
 *
 * @since 1.0.0
 * @category Pattern
 */

const time = /*#__PURE__*/(0, _function.pipe)(hrMinSecMs, /*#__PURE__*/k.or(hrMinSec), /*#__PURE__*/k.or(hrMin), k.subgroup);
/**
 * @since 1.0.0
 * @category Pattern
 */

const dateTimeStringOptT = /*#__PURE__*/(0, _function.pipe)(date, /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('T'), /*#__PURE__*/k.or( /*#__PURE__*/k.char(' ')), k.subgroup, /*#__PURE__*/k.andThen(time), k.subgroup, k.maybe)));
/**
 * @since 1.0.0
 * @category Pattern
 */

const dateTimeStringReqT = /*#__PURE__*/(0, _function.pipe)(date, /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('T'), /*#__PURE__*/k.or( /*#__PURE__*/k.char(' ')), k.subgroup)), /*#__PURE__*/k.andThen(time));
/**
 * @since 1.0.0
 * @category Pattern
 */

const isoDateStringOptTzOptT = /*#__PURE__*/(0, _function.pipe)(dateTimeStringOptT, /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)(timezoneOffset, k.maybe)));
/**
 * @since 1.0.0
 * @category Pattern
 */

const isoDateStringOptTzReqT = /*#__PURE__*/(0, _function.pipe)(dateTimeStringReqT, /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)(timezoneOffset, k.maybe)));
/**
 * @since 1.0.0
 * @category Pattern
 */

const isoDateStringReqTzReqT = /*#__PURE__*/(0, _function.pipe)(dateTimeStringReqT, /*#__PURE__*/k.andThen(timezoneOffset));
/**
 * The Date parser (used in DateFromString) accepts different strings depending on
 * runtime, and also accepts other formats like `February 29, 2022`.
 *
 * `DateFromIsoString` follows a subset of the [ECMAScript 2023 Language Date Time String
 * Specification](https://tc39.es/ecma262/#sec-date-time-string-format).
 *
 * Notable features:
 *
 * - Requires padded months, days, hours, minutes, and seconds
 * - Can be configured to require a time, time and timezone offset (e.g. `Z` or `±05:00`) or
 *   neither (default is require both).
 * - Dates may contain years, months, and days; years and months; or years
 * - Times may contain hours, minutes, seconds, and milliseconds; hours, minutes, and
 *   seconds; or hours and minutes.
 * - Expanded years are permitted (e.g. `+002022` instead of `2022`)
 *
 * @since 1.0.0
 * @category Conversion
 */

const DateFromIsoString = (params = {}) => {
  const {
    requireTime = 'TimeAndOffset'
  } = params;

  const pattern = (() => {
    switch (requireTime) {
      case 'None':
        return isoDateStringOptTzOptT;

      case 'Time':
        return isoDateStringOptTzReqT;

      case 'TimeAndOffset':
        return isoDateStringReqTzReqT;
    }
  })();

  return (0, _function.pipe)((0, _Pattern.Pattern)(pattern, requireTime === 'TimeAndOffset' ? 'IsoDateTimeStringZ' : requireTime === 'Time' ? 'IsoDateTimeString' : 'IsoDateString'), (0, _Refine.Refine)(s => !Number.isNaN(new Date(s).getTime()), 'SafeDate'), (0, _Brand.Brand)(), (0, _Imap.Imap)((0, _guardSchemable.deriveGuard)((0, _Date.Date)()), s => new Date(s), d => d.toISOString()));
};

exports.DateFromIsoString = DateFromIsoString;
//# sourceMappingURL=DateFromIsoString.js.map