import { type Branded } from 'schemata-ts/brand';
import { type Float as Floating, type MaxNegativeFloat, type MaxPositiveFloat } from 'schemata-ts/float';
import { type Schema, SchemaImplementation } from 'schemata-ts/Schema';
import { type NumberParams } from 'schemata-ts/schemables/primitives/definition';
type FloatWithDefault<Min extends number | undefined, Max extends number | undefined> = Floating<Min extends undefined ? MaxNegativeFloat : Min, Max extends undefined ? MaxPositiveFloat : Max>;
/**
 * Floating point branded newtype. Parameters: min, max are inclusive.
 *
 * Represents floating point numbers:
 *
 * ```math
 *  { f | f ∈ ℝ, f >= -Number.MAX_VALUE, f <= Number.MAX_VALUE }
 * ```
 *
 * @since 1.0.0
 * @category Number
 */
export declare const Float: <Min extends number | undefined = undefined, Max extends number | undefined = undefined>(params?: NumberParams<Min, Max> | undefined) => FloatSchema<Min, Max>;
/**
 * The FloatSchema transformer class, use instead `S.Float` function to create a FloatSchema
 *
 * @since 2.2.0
 * @category Transformations
 */
export declare class FloatSchema<Min extends number | undefined, Max extends number | undefined> extends SchemaImplementation<FloatWithDefault<Min, Max>> {
    private readonly params?;
    constructor(params?: NumberParams<Min, Max> | undefined);
    /**
     * Brands this FloatSchema with a certain brand
     *
     * @since 2.2.0
     */
    readonly brand: <Brand>() => Schema<Branded<FloatWithDefault<Min, Max>, Brand>>;
    /**
     * Sets the minimum value of the FloatSchema
     *
     * @since 2.2.0
     */
    readonly min: <NewMin extends number>(minLength: NewMin) => FloatSchema<NewMin, Max>;
    /**
     * Sets the maximum value of the FloatSchema
     *
     * @since 2.2.0
     */
    readonly max: <NewMax extends number>(maxLength: NewMax) => FloatSchema<Min, NewMax>;
    /**
     * Overrides the 'expected' field in TranscodeError > TypeMismatch
     *
     * @since 2.2.0
     * @default 'string<minLength?, maxLength?>'
     */
    readonly errorName: (errorName: string) => FloatSchema<Min, Max>;
}
export {};
//# sourceMappingURL=Float.d.ts.map