"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FloatSchema = exports.Float = void 0;

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

var _Schema = /*#__PURE__*/require("schemata-ts/Schema");

/** @since 1.0.0 */

/**
 * Floating point branded newtype. Parameters: min, max are inclusive.
 *
 * Represents floating point numbers:
 *
 * ```math
 *  { f | f ∈ ℝ, f >= -Number.MAX_VALUE, f <= Number.MAX_VALUE }
 * ```
 *
 * @since 1.0.0
 * @category Number
 */
const Float = params => new FloatSchema(params);
/**
 * The FloatSchema transformer class, use instead `S.Float` function to create a FloatSchema
 *
 * @since 2.2.0
 * @category Transformations
 */


exports.Float = Float;

class FloatSchema extends _Schema.SchemaImplementation {
  params;

  constructor(params) {
    super(s => s.float(params));
    this.params = params;
  }
  /**
   * Brands this FloatSchema with a certain brand
   *
   * @since 2.2.0
   */


  brand = () => (0, _function.unsafeCoerce)(this);
  /**
   * Sets the minimum value of the FloatSchema
   *
   * @since 2.2.0
   */

  min = min => new FloatSchema({ ...this.params,
    min
  });
  /**
   * Sets the maximum value of the FloatSchema
   *
   * @since 2.2.0
   */

  max = max => new FloatSchema({ ...this.params,
    max
  });
  /**
   * Overrides the 'expected' field in TranscodeError > TypeMismatch
   *
   * @since 2.2.0
   * @default 'string<minLength?, maxLength?>'
   */

  errorName = errorName => new FloatSchema({ ...this.params,
    errorName
  });
}

exports.FloatSchema = FloatSchema;
//# sourceMappingURL=Float.js.map