import { type Branded } from 'schemata-ts/brand';
import { type Float, type MaxNegativeFloat, type MaxPositiveFloat } from 'schemata-ts/float';
import { type Schema } from 'schemata-ts/Schema';
type FloatStringBrand<Min, Max> = {
    readonly FloatString: unique symbol;
    readonly Min: Min;
    readonly Max: Max;
};
/**
 * A string that can safely be parsed to a floating point number.
 *
 * @since 2.0.0
 */
export type FloatString<Min = MaxNegativeFloat, Max = MaxPositiveFloat> = Branded<string, FloatStringBrand<Min, Max>>;
/**
 * Floating point branded newtype from strings. Parameters: min, max are inclusive.
 *
 * Represents string floating point numbers:
 *
 * ```math
 *  { f | f ∈ ℝ, f >= -Number.MAX_VALUE, f <= Number.MAX_VALUE }
 * ```
 *
 * @deprecated Use `ParseFloat` instead.
 * @since 1.0.0
 * @category Conversion
 */
export declare const FloatFromString: Schema<FloatString, Float>;
export {};
//# sourceMappingURL=FloatFromString.d.ts.map