"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FloatFromString = void 0;

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

var k = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("kuvio"));

var _guard = /*#__PURE__*/require("schemata-ts/schemables/primitives/instances/guard");

var _Brand = /*#__PURE__*/require("schemata-ts/schemata/Brand");

var _Imap = /*#__PURE__*/require("schemata-ts/schemata/Imap");

var _Pattern = /*#__PURE__*/require("schemata-ts/schemata/Pattern");

var _Refine = /*#__PURE__*/require("schemata-ts/schemata/Refine");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/** @since 1.0.0 */

/**
 * Negative floats with at least one digit before the decimal point.
 *
 * E.g. `-1.`; but neither `-.` nor `-`
 *
 * @since 1.0.0
 * @category Pattern
 */
const negativeFloatLeft = /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('-'), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)(k.digit, /*#__PURE__*/k.between(1, 308))), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('.'), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)(k.digit, /*#__PURE__*/k.anyNumber())), k.subgroup, k.maybe)));
/**
 * Negative floats with at least one digit after the decimal point.
 *
 * E.g. `-.1`; but neither `-.` nor `-`
 *
 * @since 1.0.0
 * @category Pattern
 */

const negativeFloatRight = /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('-'), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)(k.digit, /*#__PURE__*/k.between(0, 308))), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('.'), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)(k.digit, /*#__PURE__*/k.atLeastOne())), k.subgroup, k.maybe)));
/**
 * Positive floats with at least one digit before the decimal point.
 *
 * E.g. `1.` but not `.`
 *
 * @since 1.0.0
 * @category Pattern
 */

const positiveFloatLeft = /*#__PURE__*/(0, _function.pipe)(k.digit, /*#__PURE__*/k.between(1, 308), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('.'), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)(k.digit, /*#__PURE__*/k.anyNumber())), k.subgroup, k.maybe)));
/**
 * Positive floats with at least one digit after the decimal point.
 *
 * E.g. `.1` but not `.`
 *
 * @since 1.0.0
 * @category Pattern
 */

const positiveFloatRight = /*#__PURE__*/(0, _function.pipe)(k.digit, /*#__PURE__*/k.between(0, 308), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('.'), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)(k.digit, /*#__PURE__*/k.atLeastOne())), k.subgroup, k.maybe)));
/**
 * @since 1.0.0
 * @category Pattern
 */

const decimalFloat = /*#__PURE__*/k.oneOf(negativeFloatLeft, negativeFloatRight, positiveFloatLeft, positiveFloatRight);
/**
 * @since 1.0.0
 * @category Pattern
 */

const positiveExponential = /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('+'), k.maybe, /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.integerRange(0, 308), k.subgroup)));
/**
 * @since 1.0.0
 * @category Pattern
 */

const negativeExponential = /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('-'), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)(k.digit, /*#__PURE__*/k.atLeastOne(), k.subgroup)));
/**
 * @since 1.0.0
 * @category Pattern
 */

const floatFromString = /*#__PURE__*/(0, _function.pipe)(decimalFloat, k.subgroup, /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.char('e'), /*#__PURE__*/k.andThen( /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/k.oneOf(positiveExponential, negativeExponential), k.subgroup)), k.subgroup, k.maybe)));
/**
 * Floating point branded newtype from strings. Parameters: min, max are inclusive.
 *
 * Represents string floating point numbers:
 *
 * ```math
 *  { f | f ∈ ℝ, f >= -Number.MAX_VALUE, f <= Number.MAX_VALUE }
 * ```
 *
 * @deprecated Use `ParseFloat` instead.
 * @since 1.0.0
 * @category Conversion
 */

const FloatFromString = /*#__PURE__*/(0, _function.pipe)( /*#__PURE__*/(0, _Pattern.Pattern)(floatFromString, ['FloatString', 'Float']), /*#__PURE__*/(0, _Refine.Refine)(s => _guard.PrimitivesGuard.float().is(Number(s)) && s.trim() !== '', 'Float'), /*#__PURE__*/(0, _Brand.Brand)(), /*#__PURE__*/(0, _Imap.Imap)( /*#__PURE__*/_guard.PrimitivesGuard.float(), s => Number(s), n => n.toString()));
exports.FloatFromString = FloatFromString;
//# sourceMappingURL=FloatFromString.js.map