import { type Branded } from 'schemata-ts/brand';
import { type Integer, type MaxSafeInt, type MinSafeInt } from 'schemata-ts/integer';
import { type Schema, SchemaImplementation } from 'schemata-ts/Schema';
import { type NumberParams } from 'schemata-ts/schemables/primitives/definition';
type IntWithDefaults<Min extends number | undefined, Max extends number | undefined> = Integer<Min extends undefined ? MinSafeInt : Min, Max extends undefined ? MaxSafeInt : Max>;
/**
 * Integer branded newtype. Parameters: min, max are inclusive.
 *
 * Represents integers:
 *
 * ```math
 *  { z | z ∈ ℤ, z >= -2 ** 53 + 1, z <= 2 ** 53 - 1 }
 * ```
 *
 * @since 1.0.0
 * @category Number
 */
export declare const Int: <Min extends number | undefined = undefined, Max extends number | undefined = undefined>(params?: NumberParams<Min, Max> | undefined) => IntSchema<Min, Max>;
/**
 * The IntSchema transformer class, use instead `S.Int` function to create an IntSchema
 *
 * @since 2.2.0
 * @category Transformations
 */
export declare class IntSchema<Min extends number | undefined, Max extends number | undefined> extends SchemaImplementation<IntWithDefaults<Min, Max>> {
    private readonly params?;
    constructor(params?: NumberParams<Min, Max> | undefined);
    /**
     * Brands this Int schema with a certain brand
     *
     * @since 2.2.0
     */
    readonly brand: <Brand>() => Schema<Branded<IntWithDefaults<Min, Max>, Brand>>;
    /**
     * Sets the minimum value of the IntSchema
     *
     * @since 2.2.0
     */
    readonly min: <NewMin extends number>(minLength: NewMin) => IntSchema<NewMin, Max>;
    /**
     * Sets the maximum value of the IntSchema
     *
     * @since 2.2.0
     */
    readonly max: <NewMax extends number>(maxLength: NewMax) => IntSchema<Min, NewMax>;
    /**
     * Overrides the 'expected' field in TranscodeError > TypeMismatch
     *
     * @since 2.2.0
     * @default 'string<minLength?, maxLength?>'
     */
    readonly errorName: (errorName: string) => IntSchema<Min, Max>;
}
export {};
//# sourceMappingURL=Int.d.ts.map