"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.IntSchema = exports.Int = void 0;

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

var _Schema = /*#__PURE__*/require("schemata-ts/Schema");

/** @since 1.0.0 */

/**
 * Integer branded newtype. Parameters: min, max are inclusive.
 *
 * Represents integers:
 *
 * ```math
 *  { z | z ∈ ℤ, z >= -2 ** 53 + 1, z <= 2 ** 53 - 1 }
 * ```
 *
 * @since 1.0.0
 * @category Number
 */
const Int = params => new IntSchema(params);
/**
 * The IntSchema transformer class, use instead `S.Int` function to create an IntSchema
 *
 * @since 2.2.0
 * @category Transformations
 */


exports.Int = Int;

class IntSchema extends _Schema.SchemaImplementation {
  params;

  constructor(params) {
    super(s => s.int(params));
    this.params = params;
  }
  /**
   * Brands this Int schema with a certain brand
   *
   * @since 2.2.0
   */


  brand = () => (0, _function.unsafeCoerce)(this);
  /**
   * Sets the minimum value of the IntSchema
   *
   * @since 2.2.0
   */

  min = min => new IntSchema({ ...this.params,
    min
  });
  /**
   * Sets the maximum value of the IntSchema
   *
   * @since 2.2.0
   */

  max = max => new IntSchema({ ...this.params,
    max
  });
  /**
   * Overrides the 'expected' field in TranscodeError > TypeMismatch
   *
   * @since 2.2.0
   * @default 'string<minLength?, maxLength?>'
   */

  errorName = errorName => new IntSchema({ ...this.params,
    errorName
  });
}

exports.IntSchema = IntSchema;
//# sourceMappingURL=Int.js.map