import * as k from 'kuvio';
import { type Branded } from 'schemata-ts/brand';
import { type Integer, type MaxSafeInt, type MinSafeInt } from 'schemata-ts/integer';
import { type Schema } from 'schemata-ts/Schema';
type IntStringBrand<Min, Max> = {
    readonly IntString: unique symbol;
    readonly min: Min;
    readonly max: Max;
};
/**
 * A string that can safely be parsed to a floating point number.
 *
 * @since 2.0.0
 */
export type IntString<Min = MinSafeInt, Max = MaxSafeInt> = Branded<string, IntStringBrand<Min, Max>>;
/**
 * @since 1.0.0
 * @category Pattern
 */
export declare const intFromString: k.Pattern;
/**
 * Integer branded newtype from string. Parameters: min, max are inclusive.
 *
 * Note: has an optional `encodeToBase` parameter that controls the output base of the
 * encoded string. Currently only decodes binary, octal, decimal, and hexadecimal input
 * bases. It decodes in any base, and encodes to supplied parameter defaulting to decimal.
 *
 * _Note_: Does not currently allow exponential notation (e.g. `0x123e4`).
 *
 * Represents string-integers:
 *
 * ```math
 *  { z | z ∈ ℤ, z >= -2 ** 53 + 1, z <= 2 ** 53 - 1 }
 * ```
 *
 * @deprecated Use `ParseInt` instead.
 * @since 1.0.0
 * @category Conversion
 */
export declare const IntFromString: Schema<IntString, Integer>;
export {};
//# sourceMappingURL=IntFromString.d.ts.map