import { type Branded } from 'schemata-ts/brand';
import { type Schema, SchemaImplementation } from 'schemata-ts/Schema';
import { type StringParams } from 'schemata-ts/schemables/primitives/definition';
/**
 * Represents string inputs / outputs
 *
 * @since 1.0.0
 * @category String
 */
export declare const String: (params?: StringParams) => StringSchema;
/**
 * The StringSchema transformer class, use instead `S.String` function to create a StringSchema
 *
 * @since 2.2.0
 * @category Transformations
 */
export declare class StringSchema extends SchemaImplementation<string> {
    private readonly params?;
    constructor(params?: StringParams | undefined);
    /**
     * Brands this string schema with a certain brand
     *
     * @since 2.2.0
     */
    readonly brand: <Brand>() => Schema<Branded<string, Brand>>;
    /**
     * Sets the minimum required length of the string
     *
     * @since 2.2.0
     */
    readonly minLength: (minLength: number) => StringSchema;
    /**
     * Sets the maximum required length of the string
     *
     * @since 2.2.0
     */
    readonly maxLength: (maxLength: number) => StringSchema;
    /**
     * Overrides the 'expected' field in TranscodeError > TypeMismatch
     *
     * @since 2.2.0
     * @default 'string<minLength?, maxLength?>'
     */
    readonly errorName: (errorName: string) => StringSchema;
}
//# sourceMappingURL=String.d.ts.map