"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.StringSchema = exports.String = void 0;

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

var _Schema = /*#__PURE__*/require("schemata-ts/Schema");

/** @since 1.0.0 */

/**
 * Represents string inputs / outputs
 *
 * @since 1.0.0
 * @category String
 */
const String = params => new StringSchema(params);
/**
 * The StringSchema transformer class, use instead `S.String` function to create a StringSchema
 *
 * @since 2.2.0
 * @category Transformations
 */


exports.String = String;

class StringSchema extends _Schema.SchemaImplementation {
  params;

  constructor(params) {
    super(s => s.string(params));
    this.params = params;
  }
  /**
   * Brands this string schema with a certain brand
   *
   * @since 2.2.0
   */


  brand = () => (0, _function.unsafeCoerce)(this);
  /**
   * Sets the minimum required length of the string
   *
   * @since 2.2.0
   */

  minLength = minLength => new StringSchema({ ...this.params,
    minLength
  });
  /**
   * Sets the maximum required length of the string
   *
   * @since 2.2.0
   */

  maxLength = maxLength => new StringSchema({ ...this.params,
    maxLength
  });
  /**
   * Overrides the 'expected' field in TranscodeError > TypeMismatch
   *
   * @since 2.2.0
   * @default 'string<minLength?, maxLength?>'
   */

  errorName = errorName => new StringSchema({ ...this.params,
    errorName
  });
}

exports.StringSchema = StringSchema;
//# sourceMappingURL=String.js.map