"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.StructSchema = exports.Struct = void 0;

var _function = /*#__PURE__*/require("fp-ts/lib/function.js");

var O = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/Option.js"));

var RR = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/ReadonlyRecord.js"));

var Sg = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("fp-ts/lib/Semigroup.js"));

var _guardSchemable = /*#__PURE__*/require("schemata-ts/derivations/guard-schemable");

var _informationSchemable = /*#__PURE__*/require("schemata-ts/derivations/information-schemable");

var _typeStringSchemable = /*#__PURE__*/require("schemata-ts/derivations/type-string-schemable");

var _util = /*#__PURE__*/require("schemata-ts/internal/util");

var _Schema = /*#__PURE__*/require("schemata-ts/Schema");

var _typeString = /*#__PURE__*/require("schemata-ts/schemables/struct/instances/type-string");

var _Imap = /*#__PURE__*/require("schemata-ts/schemata/Imap");

var _Option2 = /*#__PURE__*/require("schemata-ts/schemata/Option");

var _Optional = /*#__PURE__*/require("schemata-ts/schemata/Optional");

var _Readonly = /*#__PURE__*/require("schemata-ts/schemata/Readonly");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/** @since 1.4.0 */

/**
 * Used to construct a struct schema with enumerated keys.
 *
 * **Note:** Index signatures must accomodate the input/output types for all other
 * enumerated keys. It will decode properly otherwise, but TypeScript will not permit
 * construction of such a type
 *
 * **Note:** The second parameter `extraProps` is deprecated, use `Struct({}).strict()` or
 * `Struct({}).addIndexSignature()` instead
 *
 * @since 1.0.0
 * @category Combinators
 * @see https://www.typescriptlang.org/docs/handbook/2/objects.html#index-signatures
 */
const Struct = (props, extraProps = 'strip') => new StructSchema(props, extraProps);
/**
 * Use `Struct({})` schema combinator instead
 *
 * @since 2.1.0
 * @category Transformations
 */


exports.Struct = Struct;

class StructSchema extends _Schema.SchemaImplementation {
  definition;
  indexSignature;

  constructor(definition, indexSignature) {
    super(_ => {
      const struct = {};
      const structName = {};

      for (const key in definition) {
        if (!(0, _util.hasOwn)(definition, key)) continue; // eslint-disable-next-line @typescript-eslint/no-non-null-assertion

        const schema = definition[key];
        const schemable = schema.runSchema(_);
        const guard = (0, _guardSchemable.deriveGuard)(schema);
        const information = (0, _informationSchemable.deriveInformation)(schema);
        const name = (0, _typeStringSchemable.deriveTypeString)(schema);
        struct[key] = {
          schemable,
          guard,
          information,
          semigroup: Sg.last(),
          name: name[0]
        };
        structName[key] = {
          schemable: name,
          guard,
          information,
          semigroup: Sg.last(),
          name: name[0]
        };
      }

      const extraProps_ = indexSignature === undefined || typeof indexSignature === 'string' ? indexSignature : indexSignature.runSchema(_);
      const extraPropsName_ = indexSignature === undefined || typeof indexSignature === 'string' ? indexSignature : (0, _typeStringSchemable.deriveTypeString)(indexSignature);

      const wholeName = _typeString.StructTypeString.struct(structName, extraPropsName_, '');

      return _.struct(struct, extraProps_, wholeName[0]);
    });
    this.definition = definition;
    this.indexSignature = indexSignature;
  }
  /**
   * Re-declares a StructSchema by only including specified properties.
   *
   * Returns a new `StructSchema`
   *
   * @since 2.1.0
   */


  pick = (...keys) => new StructSchema((0, _function.pipe)(this.definition, RR.filterWithIndex(k => keys.includes(k)), _ => (0, _function.unsafeCoerce)(_)), this.indexSignature);
  /**
   * Re-declares a StructSchema by excluding specified properties.
   *
   * Returns a new `StructSchema`
   *
   * @since 2.1.0
   */

  omit = (...keys) => new StructSchema((0, _function.pipe)(this.definition, RR.filterWithIndex(k => !keys.includes(k)), _ => (0, _function.unsafeCoerce)(_)), this.indexSignature);
  /**
   * Marks all properties as optional; applies `Partial` to both input and output types.
   *
   * Returns a new schema.
   *
   * @since 2.1.0
   */

  partial = () => (0, _function.unsafeCoerce)(new StructSchema((0, _function.pipe)(this.definition, RR.map(_Optional.Optional)), this.indexSignature));
  /**
   * A variant of `partial` that applies `Partial` to input properties and maps each
   * output property to the fp-ts `Option` type.
   *
   * Returns a new schema.
   *
   * @since 2.1.0
   */

  partialOption = () => (0, _function.unsafeCoerce)(new StructSchema((0, _function.pipe)(this.definition, RR.map(OptionFromOptional)), this.indexSignature));
  /**
   * Marks all properties as readonly; applies `Readonly` to both input and output types.
   *
   * Returns a new schema.
   *
   * @since 2.1.0
   */

  readonly = () => (0, _Readonly.Readonly)(new StructSchema(this.definition, this.indexSignature));
  /**
   * Sets a Struct Schema's index signature to be strict
   *
   * Returns a new `StructSchema`.
   *
   * @since 2.1.0
   */

  strict = () => new StructSchema(this.definition, 'error');
  /**
   * Adds an index signature to a Struct Schema.
   *
   * Returns a new `StructSchema`.
   *
   * @since 2.1.0
   */

  addIndexSignature = indexSignature => new StructSchema(this.definition, indexSignature);
  /**
   * Extends a Struct Schema with additional properties. Keys specified in `props` will
   * overwrite keys in `this`.
   *
   * Returns a new `StructSchema`.
   *
   * @since 2.1.0
   */

  extend = props => this.intersect(new StructSchema(props, this.indexSignature));
  /**
   * Intersects the present Struct Schema with another effectively concatenating their
   * keys. Keys in `this` will be overwritten with identical keys in `that`.
   *
   * **Note:** The index signature of `that` will be discarded.
   *
   * Returns a new `StructSchema`.
   *
   * @since 2.1.0
   */

  intersect = that => new StructSchema({ ...this.definition,
    ...that.definition
  }, this.indexSignature);
}

exports.StructSchema = StructSchema;

const OptionFromOptional = schema => {
  const guard = (0, _guardSchemable.deriveGuard)((0, _Option2.Option)(schema));
  return (0, _function.pipe)((0, _Optional.Optional)(schema), (0, _Imap.Imap)({
    is: u => guard.is(u) && (0, _function.pipe)(u, O.fold(() => true, a => a !== undefined))
  }, O.fromNullable, O.toUndefined));
};
//# sourceMappingURL=Struct.js.map