import { type InputOf, type OutputOf, type Schema, SchemaImplementation } from 'schemata-ts/Schema';
/**
 * A schema for n-tuples
 *
 * @since 1.0.0
 * @category Combinators
 */
export declare const Tuple: <T extends readonly Schema<any, any>[]>(...items: T) => TupleSchema<T>;
type Input<T extends ReadonlyArray<Schema<any, any>>> = {
    readonly [K in keyof T]: InputOf<T[K]>;
};
type Output<T extends ReadonlyArray<Schema<any, any>>> = {
    readonly [K in keyof T]: OutputOf<T[K]>;
};
/**
 * The TupleSchema transformer class, use instead `Tuple` function
 *
 * @since 2.2.0
 * @category Transformations
 */
export declare class TupleSchema<T extends ReadonlyArray<Schema<any>>> extends SchemaImplementation<Input<T>, Output<T>> {
    private readonly items;
    constructor(items: T);
    /**
     * Appends items to the end of the tuple
     *
     * @since 2.2.0
     */
    readonly append: <U extends readonly Schema<any, any>[]>(...items: U) => TupleSchema<readonly [...T, ...U]>;
    /**
     * Prepends items to the beginning of the tuple
     *
     * @since 2.2.0
     */
    readonly prepend: <U extends readonly Schema<any, any>[]>(...items: U) => TupleSchema<readonly [...U, ...T]>;
}
export {};
//# sourceMappingURL=Tuple.d.ts.map