"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TupleSchema = exports.Tuple = void 0;

var _typeStringSchemable = /*#__PURE__*/require("schemata-ts/derivations/type-string-schemable");

var _Schema = /*#__PURE__*/require("schemata-ts/Schema");

var _typeString = /*#__PURE__*/require("schemata-ts/schemables/array/instances/type-string");

/** @since 1.4.0 */

/**
 * A schema for n-tuples
 *
 * @since 1.0.0
 * @category Combinators
 */
const Tuple = (...items) => new TupleSchema(items);
/**
 * The TupleSchema transformer class, use instead `Tuple` function
 *
 * @since 2.2.0
 * @category Transformations
 */


exports.Tuple = Tuple;

class TupleSchema extends _Schema.SchemaImplementation {
  items;

  constructor(items) {
    const name = _typeString.ArrayTypeString.tuple('', ...items.map(_typeStringSchemable.deriveTypeString))[0];

    super(s => s.tuple(name, ...items.map(schema => schema.runSchema(s))));
    this.items = items;
  }
  /**
   * Appends items to the end of the tuple
   *
   * @since 2.2.0
   */


  append = (...items) => new TupleSchema([...this.items, ...items]);
  /**
   * Prepends items to the beginning of the tuple
   *
   * @since 2.2.0
   */

  prepend = (...items) => new TupleSchema([...items, ...this.items]);
}

exports.TupleSchema = TupleSchema;
//# sourceMappingURL=Tuple.js.map