/** @since 1.0.0 */
import { type Integer, type MaxSafeInt, maxSafeInt } from 'schemata-ts/integer'
import { type Schema } from 'schemata-ts/Schema'
import { Int } from 'schemata-ts/schemata/Int'

/**
 * Positive integer branded newtype.
 *
 * Represents positive integers:
 *
 * ```math
 *  { z | z ∈ ℤ, z > 0, z <= 2 ** 53 - 1 }
 * ```
 *
 * @since 1.0.0
 * @category Number
 */
export const PositiveInt: Schema<Integer<1, MaxSafeInt>> = Int({
  min: 1,
  max: maxSafeInt,
})
